#!/usr/bin/env python3

"""Provide a configuration dialog for the dock panel applet

    Allow the user to view and set various configuration options
"""
# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
else:
    gi.require_version("Gtk", "3.0")

gi.require_version("MatePanelApplet", "4.0")

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import MatePanelApplet
from gi.repository import GdkPixbuf

from random import randint

from docked_app_helpers import *


class AttentionType:
    """Class to specify the ways in which docked apps can shoe that they need attention
    """
    BLINK = 0
    SHOW_BADGE = 1


class ClickActionType:
    """ Class to define the ways in which running apps may perorm window selection"""

    WIN_LIST = 0  # show the applet's window list
    COMPIZ = 1    # display compiz window spread
    MINMAX = 2    # minimize/maximize the app's open windows


class ThemeType:
    """ Class to define the various themes that may be set """
    DEFAULT = 0
    UNITY = 1
    UNITY_FLAT = 2
    SUBWAY = 3
    CUSTOM = 4


def create_frame(caption):
    """ Convenience function to create a Gtk.Frame with a desired caption
        in bold text

    Returns:
        frame - the Gtk.Frame we created

    """

    frame = Gtk.Frame(label="aaaa")
    lbl = frame.get_label_widget()
    lbl.set_use_markup(True)
    lbl.set_label("<b>%s</b>" % caption)
    frame.set_shadow_type(Gtk.ShadowType.NONE)
    frame.set_border_width(4)
    return frame


class DockPrefsWindow(Gtk.Window):
    """Class to provide the preferences window functionality

    Create and display the preferences window

    Provide methods to get and set:
        the type of indicator to be used by the dock applet
        whether pinned/unpinned apps are to be displayed from all workspaces or
        just the current workspace
        whether the colour of the MATE panels is to be set to the dominant
         colour of the current wallpaper image

    """

    def __init__(self, ok_callback, app):
        """ Constructor for the preferences window

        Create the window and its contents and display them

        set the callback for the ok button press

        Args:
            ok_callback : the method to be called when the ok button is
                          is pressed
            app         : a docked_app from which we can generate previews of the
                          various indicator and background settings
            preview_size : the size of preview of indicator and background
                           settings

        """

        super().__init__(title=_("Preferences"))

        self.set_skip_taskbar_hint(True)  # we don't want to be in the taskbar
        self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)

        # define the size of the preview window drawing area. It should match_bamf_app_to_dock_app
        # the size of the dock apps drawing areas
        self.PREVIEW_SIZE = app.drawing_area_size

        self.connect("delete-event", self.win_delete_event)

        self.__app = app

        self.__app_pb = app.app_pb

        # setup the window contents
        self.set_border_width(5)
        if build_gtk2:
            self.__hbox = Gtk.HBox()
        else:
            self.__hbox = Gtk.Box()
            self.__hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.__hbox.set_spacing(2)

        if build_gtk2:
            self.__hbox1 = Gtk.HBox()
        else:
            self.__hbox1 = Gtk.Box()
            self.__hbox1.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.__hbox1.set_spacing(2)

        self.__vbox = self.create_vbox()
        self.__vbox.set_spacing(2)

        self.__cancel_btn = Gtk.Button(label=_("Cancel"),
                                       stock=Gtk.STOCK_CANCEL)
        self.__cancel_btn.connect("button-press-event",
                                  self.win_cancel_button_press)
        self.__ok_btn = Gtk.Button(label=_("Ok"), stock=Gtk.STOCK_OK)
        self.__ok_btn.connect("button-press-event", ok_callback)

        if build_gtk2:
            self.__hbbx = Gtk.HButtonBox()
        else:
            self.__hbbx = Gtk.ButtonBox()
            self.__hbbx.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.__hbbx.set_spacing(4)
        self.__hbbx.set_layout(Gtk.ButtonBoxStyle.END)

        self.__hbbx.pack_start(self.__ok_btn, False, False, 4)
        self.__hbbx.pack_start(self.__cancel_btn, False, False, 4)
        self.__notebook = Gtk.Notebook()

        if build_gtk2:
            self.__appearance_tbl = Gtk.Table(rows=4, columns=1,
                                              homogeneous=False)
        else:
            self.__appearance_tbl = Gtk.Grid()
            self.__appearance_tbl.set_column_spacing(4)
            self.__appearance_tbl.set_row_spacing(4)
            self.__appearance_tbl.set_row_homogeneous(False)

        self.__frame_theme = create_frame(_("Theme"))
        self.__frame_theme_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                 xscale=1.0, yscale=1.0)
        self.__frame_theme_align.set_padding(0, 0, 12, 0)
        self.__frame_theme_align.set_padding(0, 0, 12, 0)

        self.__theme_options = [_("Default"), _("Unity"), _("Unity Flat"), _("Subway"), _("Custom")]
        self.__cbt_theme = self.create_textcombo(self.__theme_options)
        self.__cbt_theme.connect("changed", self.theme_changed)
        self.__frame_theme_align.add(self.__cbt_theme)
        self.__frame_theme.add(self.__frame_theme_align)

        self.__frame_ind_type = create_frame(_("Indicator Type"))
        self.__indicator_options = [_("Default light"), _("Default dark"), _("Single bar"),
                                    _("Circle"), _("Square"), _("Triangle"), _("Diamond"),
                                    _("Subway"), _("None")]
        self.__cbt_ind_type = self.create_textcombo(self.__indicator_options)
        self.__cbt_ind_type.connect("changed", self.setting_toggled)

        self.__frame_preview = create_frame(_("Preview"))
        self.__frame_preview.set_shadow_type(Gtk.ShadowType.NONE)
        self.__hbox_preview = self.create_hbox()
        self.__hbox_preview.set_spacing(0)

        self.__da_preview = Gtk.DrawingArea()
        self.__da_preview.set_size_request(self.PREVIEW_SIZE * 3, self.PREVIEW_SIZE)

        self.__frame_preview_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                   xscale=1.0, yscale=1.0)
        self.__frame_preview_align.set_padding(0, 0, 12, 0)
        self.__hbox_preview.pack_start(self.__da_preview, False, False, 0)
        self.__frame_preview_align.add(self.__hbox_preview)
        self.__vbox_preview = self.create_vbox()
        self.__vbox.set_spacing(2)
        self.__vbox_preview.pack_start(self.__frame_preview_align, False, False, 4)
        self.__frame_preview.add(self.__vbox_preview)

        # connect an event handler to draw the dark indicator
        if build_gtk2:
            self.__da_preview.connect("expose-event", self.draw_preview)
        else:
            self.__da_preview.connect("draw", self.draw_preview)

        # create ui elements for multiple indicators for open windows
        self.__cb_multi_ind = Gtk.CheckButton(label=_("Display an indicator for each open window"))
        self.__cb_multi_ind.set_tooltip_text(_("Display an indicator (max 4) for each open window"))
        self.__cb_multi_ind.connect("toggled", self.setting_toggled)

        if build_gtk2:
            self.__tbl_ind_type = Gtk.Table(rows=1, columns=2,
                                            homogeneous=False)
            self.__tbl_ind_type.attach(self.__cbt_ind_type,
                                       0, 1, 0, 1,
                                       Gtk.AttachOptions.FILL,
                                       Gtk.AttachOptions.SHRINK,
                                       2, 2)

        else:
            self.__tbl_ind_type = Gtk.Grid()
            self.__tbl_ind_type.set_row_spacing(2)
            self.__tbl_ind_type.set_column_spacing(2)
            self.__tbl_ind_type.attach(self.__cbt_ind_type, 0, 0, 1, 1)

        self.__frame_ind_type_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                    xscale=1.0, yscale=1.0)
        self.__frame_ind_type_align.set_padding(0, 0, 12, 0)
        self.__frame_ind_type_align.add(self.__tbl_ind_type)
        self.__frame_ind_type.add(self.__frame_ind_type_align)

        self.__frame_bg = create_frame(_("Icon Background"))
        self.__bg_options = [_("Gradient fill"), _("Solid fill"), _("Unity"), _("Unity Flat")]
        self.__cbt_icon_bg = self.create_textcombo(self.__bg_options)
        self.__cbt_icon_bg.connect("changed", self.setting_toggled)

        self.__frame_bg_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                              xscale=1.0, yscale=1.0)
        self.__frame_bg_align.set_padding(0, 0, 12, 0)
        self.__frame_bg_align.add(self.__cbt_icon_bg)
        self.__frame_bg.add(self.__frame_bg_align)

        if build_gtk2:
            self.__appearance_tbl.attach(self.__frame_preview, 0, 1, 0, 1,
                                         Gtk.AttachOptions.FILL,
                                         Gtk.AttachOptions.SHRINK,
                                         2, 2)
            self.__appearance_tbl.attach(self.__frame_ind_type, 0, 1, 1, 2,
                                         Gtk.AttachOptions.FILL,
                                         Gtk.AttachOptions.SHRINK,
                                         2, 2)
            self.__appearance_tbl.attach(self.__cb_multi_ind, 0, 1, 2, 3,
                                         Gtk.AttachOptions.FILL,
                                         Gtk.AttachOptions.SHRINK,
                                         2, 2)
            self.__appearance_tbl.attach(self.__frame_bg, 0, 1, 3, 4,
                                         Gtk.AttachOptions.FILL,
                                         Gtk.AttachOptions.SHRINK,
                                         2, 2)

        else:
            self.__appearance_tbl.attach(self.__frame_preview, 0, 0, 1, 1)
            self.__appearance_tbl.attach(self.__frame_theme, 0, 1, 1, 1)

            #self.__appearance_tbl.attach(self.__frame_them_fb_bar_col, 0, 1, 1, 1)
            self.__appearance_tbl.attach(self.__frame_ind_type, 0, 2, 1, 1)
            self.__appearance_tbl.attach(self.__cb_multi_ind, 0, 3, 1, 1)
            self.__appearance_tbl.attach(self.__frame_bg, 0, 4, 1, 1)

        self.__frame_pinned_apps = create_frame(_("Pinned application dock icons"))
        self.__rb_pinned_all_ws = Gtk.RadioButton(label=_("Display on all workspaces"))
        self.__rb_pinned_pin_ws = Gtk.RadioButton(label=_("Display only on the workspace the app was pinned"),
                                                  group=self.__rb_pinned_all_ws)

        if build_gtk2:
            self.__table_pinned_apps = Gtk.Table(rows=2, columns=1,
                                                 homogeneous=False)
            self.__table_pinned_apps.attach(self.__rb_pinned_all_ws,
                                            0, 1, 0, 1,
                                            Gtk.AttachOptions.FILL,
                                            Gtk.AttachOptions.SHRINK,
                                            2, 2)
            self.__table_pinned_apps.attach(self.__rb_pinned_pin_ws,
                                            0, 1, 1, 2,
                                            Gtk.AttachOptions.FILL,
                                            Gtk.AttachOptions.SHRINK,
                                            2, 2)
        else:
            self.__table_pinned_apps = Gtk.Grid()
            self.__table_pinned_apps.set_row_spacing(2)
            self.__table_pinned_apps.set_column_spacing(2)
            self.__table_pinned_apps.attach(self.__rb_pinned_all_ws,
                                            0, 0, 1, 1)
            self.__table_pinned_apps.attach(self.__rb_pinned_pin_ws,
                                            0, 1, 1, 1)

        self.__frame_pinned_apps_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                       xscale=1.0, yscale=1.0)
        self.__frame_pinned_apps_align.set_padding(0, 0, 12, 0)
        self.__frame_pinned_apps_align.add(self.__table_pinned_apps)
        self.__frame_pinned_apps.add(self.__frame_pinned_apps_align)

        self.__frame_unpinned_apps = create_frame(_("Unpinned application dock icons"))
        self.__rb_unpinned_all_ws = Gtk.RadioButton(label=_("Display unpinned apps from all workspaces"))
        self.__rb_unpinned_cur_ws = Gtk.RadioButton(group=self.__rb_unpinned_all_ws,
                                                    label=_("Display unpinned apps only from current workspace"))

        if build_gtk2:
            self.__table_unpinned_apps = Gtk.Table(rows=2, columns=1,
                                                   homogeneous=False)
            self.__table_unpinned_apps.attach(self.__rb_unpinned_all_ws,
                                              0, 1, 0, 1,
                                              Gtk.AttachOptions.FILL,
                                              Gtk.AttachOptions.SHRINK,
                                              2, 2)
            self.__table_unpinned_apps.attach(self.__rb_unpinned_cur_ws,
                                              0, 1, 1, 2,
                                              Gtk.AttachOptions.FILL,
                                              Gtk.AttachOptions.SHRINK,
                                              2, 2)
        else:
            self.__table_unpinned_apps = Gtk.Grid()
            self.__table_unpinned_apps.set_row_spacing(2)
            self.__table_unpinned_apps.set_column_spacing(2)
            self.__table_unpinned_apps.attach(self.__rb_unpinned_all_ws,
                                              0, 0, 1, 1)
            self.__table_unpinned_apps.attach(self.__rb_unpinned_cur_ws,
                                              0, 1, 1, 1)

        self.__frame_unpinned_apps_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                         xscale=1.0, yscale=1.0)
        self.__frame_unpinned_apps_align.set_padding(0, 0, 12, 0)
        self.__frame_unpinned_apps_align.add(self.__table_unpinned_apps)
        self.__frame_unpinned_apps.add(self.__frame_unpinned_apps_align)

        self.__cb_win_cur_ws = Gtk.CheckButton(label=_("Display indicators/window list items for current workspace only"))

        self.__notes_scrolled_win = Gtk.ScrolledWindow()
        self.__notes_scrolled_win.set_policy(Gtk.PolicyType.NEVER,
                                             Gtk.PolicyType.AUTOMATIC)
        self.__tv_notes = Gtk.TextView()
        self.__tv_notes.set_wrap_mode(Gtk.WrapMode.WORD)
        self.__tv_notes.set_editable(False)
        self.__notes_text_buf = Gtk.TextBuffer()

        self.__tv_notes.set_buffer(self.__notes_text_buf)
        iter = self.__notes_text_buf.get_start_iter()
        self.__notes_text_buf.insert(iter,
                                     _("Note: when displaying pinned apps only on the workspace where they were " +
                                     "created, it is a good idea to also select the 'Display unpinned apps' " +
                                     "and 'Display indicators/window list' items for the current workspace " +
                                     "only options."))

        self.__ws_vbox = self.create_vbox()
        self.__ws_vbox.set_spacing(2)
        self.__ws_vbox.pack_start(self.__frame_pinned_apps, False, False, 4)
        self.__ws_vbox.pack_start(self.__frame_unpinned_apps, False, False, 4)
        self.__ws_vbox.pack_start(self.__cb_win_cur_ws, False, False, 4)
        self.__ws_vbox.pack_end(self.__tv_notes, False, False, 4)

        self.__frame_behaviour = create_frame(_("Left clicking a running app's icon will:"))
        self.__rb_win_list = Gtk.RadioButton(label=_("Display a list of the app's windows"))
        self.__rb_win_thumb = Gtk.RadioButton(group=self.__rb_win_list,
                                              label=_("Show thumbnail previews of the app's windows"))
        self.__rb_win_minmax = Gtk.RadioButton(group=self.__rb_win_list,
                                               label=_("Minimize/restore all of the app's windows"))
        self.__table_behaviour = Gtk.Table(rows=3, columns=1,
                                           homogeneous=False)
        self.__table_behaviour.attach(self.__rb_win_list,
                                      0, 1, 0, 1,
                                      Gtk.AttachOptions.FILL,
                                      Gtk.AttachOptions.SHRINK,
                                      2, 2)
        self.__table_behaviour.attach(self.__rb_win_thumb,
                                      0, 1, 1, 2,
                                      Gtk.AttachOptions.FILL,
                                      Gtk.AttachOptions.SHRINK,
                                      2, 2)
        self.__table_behaviour.attach(self.__rb_win_minmax,
                                      0, 1, 2, 3,
                                      Gtk.AttachOptions.FILL,
                                      Gtk.AttachOptions.SHRINK,
                                      2, 2)

        self.__cb_win_switch_unity_style = Gtk.CheckButton(label=_("Unity-style behaviour (always focus app on first click)"))

        self.__notes_behaviour_tv = Gtk.TextView()
        self.__notes_behaviour_tv.set_wrap_mode(Gtk.WrapMode.WORD)
        self.__notes_behaviour_tv.set_editable(False)
        self.__notes_behaviour_tb = Gtk.TextBuffer()

        self.__notes_behaviour_tv.set_buffer(self.__notes_behaviour_tb)
        iter = self.__notes_behaviour_tb.get_start_iter()
        self.__notes_behaviour_tb.insert(iter,
                                         _("\nNotes:\nIf an app has only a single window open, a window list will not be " +
                                         "displayed. Instead the window will be minimized/restored.\n" +
                                         "Window thumbnail previews require Compiz"))
        self.__frame_behaviour_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                     xscale=1.0, yscale=1.0)
        self.__frame_behaviour_align.set_padding(0, 0, 12, 0)
        self.__frame_behaviour_align.add(self.__table_behaviour)

        self.__frame_behaviour.add(self.__frame_behaviour_align)

        self.__behaviour_vbox = self.create_vbox()
        self.__behaviour_vbox.set_spacing(2)
        self.__behaviour_vbox.pack_start(self.__frame_behaviour, False,
                                         False, 4)
        self.__behaviour_vbox.pack_start(self.__cb_win_switch_unity_style, False, False, 4)
        self.__behaviour_vbox.pack_start(self.__notes_behaviour_tv, False, False, 4)

        self.__frame_spc = create_frame(_("App spacing"))
        if build_gtk2:
            self.__sb_spc = Gtk.SpinButton()
            self.__sb_spc.set_adjustment(Gtk.Adjustment(0, 0, 7, 1, 4))
        else:
            self.__sb_spc = Gtk.SpinButton.new_with_range(0, 7, 1)

        self.__sb_spc.set_update_policy(Gtk.SpinButtonUpdatePolicy.IF_VALID)
        self.__sb_spc.set_numeric(True)
        self.__sb_spc.set_max_length(1)
        self.__sb_spc.set_snap_to_ticks(True)

        self.__frame_spc_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                               xscale=1.0, yscale=1.0)
        self.__frame_spc_align.set_padding(0, 0, 12, 300)
        self.__frame_spc_align.add(self.__sb_spc)
        self.__frame_spc.add(self.__frame_spc_align)

        self.__frame_color_change = create_frame(_("Panel colour"))
        self.__cb_panel_color_change = Gtk.CheckButton(label=_("Change panel colour to match wallpaper"))
        self.__cb_panel_color_change.connect("toggled", self.color_change_toggled)
        self.__cb_dock_panel_only = Gtk.CheckButton(label=_("Change colour of dock's panel only"))

        if build_gtk2:
            self.__table_color_change = Gtk.Table(rows=3, columns=1,
                                                  homogeneous=False)
            self.__table_color_change.attach(self.__cb_panel_color_change,
                                             0, 1, 0, 1,
                                             Gtk.AttachOptions.FILL,
                                             Gtk.AttachOptions.SHRINK,
                                             2, 2)
            self.__table_color_change.attach(self.__cb_dock_panel_only,
                                             0, 1, 1, 2,
                                             Gtk.AttachOptions.FILL,
                                             Gtk.AttachOptions.SHRINK,
                                             2, 2)

        else:
            self.__table_color_change = Gtk.Grid()
            self.__table_color_change.set_row_spacing(2)
            self.__table_color_change.set_column_spacing(2)

            self.__table_color_change.attach(self.__cb_panel_color_change,
                                             0, 0, 1, 1)
            self.__table_color_change.attach(self.__cb_dock_panel_only,
                                             0, 1, 1, 1)

        self.__frame_color_change_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                        xscale=1.0, yscale=1.0)
        self.__frame_color_change_align.set_padding(0, 0, 12, 0)
        self.__frame_color_change_align.add(self.__table_color_change)
        self.__frame_color_change.add(self.__frame_color_change_align)

        if not build_gtk2:
            self.__frame_dock_size = create_frame(_("Dock size"))
            self.__rb_variable_ds = Gtk.RadioButton(label=_("Variable - expand or contract as necessary"))
            self.__rb_fixed_ds = Gtk.RadioButton(group=self.__rb_variable_ds,
                                                 label=_("Fixed"))

            self.__lbl_fixed_size = Gtk.Label(_("Display up to "))

            self.__sb_fixed_size = Gtk.SpinButton.new_with_range(2, 64, 1)
            self.__sb_fixed_size.set_update_policy(Gtk.SpinButtonUpdatePolicy.IF_VALID)
            self.__sb_fixed_size.set_numeric(True)
            self.__sb_fixed_size.set_max_length(2)
            self.__sb_fixed_size.set_snap_to_ticks(True)

            self.__lbl_fixed_size1 = Gtk.Label(_(" app icons"))

            self.__hbox_fixed_size = self.create_hbox()
            self.__hbox_fixed_size.set_spacing(2)
            self.__hbox_fixed_size.pack_start(self.__lbl_fixed_size, False, False, 4)
            self.__hbox_fixed_size.pack_start(self.__sb_fixed_size, False, False, 4)
            self.__hbox_fixed_size.pack_start(self.__lbl_fixed_size1, False, False, 4)

            self.__table_dock_size = Gtk.Grid()
            self.__table_dock_size.set_row_spacing(2)
            self.__table_dock_size.set_column_spacing(2)

            self.__table_dock_size.attach(self.__rb_variable_ds,
                                          0, 0, 1, 1)
            self.__table_dock_size.attach(self.__rb_fixed_ds,
                                          0, 1, 1, 1)
            self.__table_dock_size.attach(self.__hbox_fixed_size, 0, 2, 1, 1)

            self.__frame_dock_size_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                         xscale=1.0, yscale=1.0)
            self.__frame_dock_size_align.set_padding(0, 0, 12, 0)
            self.__frame_dock_size_align.add(self.__table_dock_size)
            self.__frame_dock_size.add(self.__frame_dock_size_align)

        self.__cb_pan_act = Gtk.CheckButton(label=_("Disable popup action list " +
                                            "and show app actions\non panel " +
                                            "right click menu only"))

        self.__panel_vbox = self.create_vbox()
        self.__panel_vbox.set_spacing(2)
        self.__panel_vbox.pack_start(self.__frame_spc, False, False, 4)
        self.__panel_vbox.pack_start(self.__frame_color_change,
                                     False, False, 4)
        if not build_gtk2:
            self.__panel_vbox.pack_start(self.__frame_dock_size,
                                         False, False, 4)

        self.__panel_vbox.pack_start(self.__cb_pan_act,
                                     False, False, 4)

        self.__frame_fb_bar_col = create_frame(_("Fallback bar indicator colour"))
        self.__lbl_fb_bar_col = Gtk.Label(_("Colour"))
        self.__cbtn_fb_bar_col = Gtk.ColorButton()
        self.__cbtn_fb_bar_col.set_tooltip_text(_("Colour used for drawing bar indicators when theme colour " +
                                                "cannot be determined or when using Gtk2"))

        self.__fb_bar_col_hbox = self.create_hbox()
        self.__fb_bar_col_hbox.set_spacing(2)
        self.__fb_bar_col_hbox.pack_start(self.__lbl_fb_bar_col, False, False, 2)
        self.__fb_bar_col_hbox.pack_start(self.__cbtn_fb_bar_col, True, True, 2)
        self.__fb_bar_col_vbox = self.create_vbox()
        self.__fb_bar_col_vbox.set_spacing(2)
        self.__fb_bar_col_vbox.pack_start(self.__fb_bar_col_hbox, False, False, 2)
        self.__frame_fb_bar_col.add(self.__fb_bar_col_vbox)

        self.__frame_attention_type = create_frame(_("Action when apps need attention"))
        self.__rb_attention_blink = Gtk.RadioButton(label=_("Blink the app icon"))
        self.__rb_attention_badge = Gtk.RadioButton(label=_("Show a badge on the app icon"),
                                                    group=self.__rb_attention_blink)

        if build_gtk2:
            self.__table_attention_type = Gtk.Table(rows=2, columns=1,
                                                    homogeneous=False)
            self.__table_attention_type.attach(self.__rb_attention_blink,
                                               0, 1, 0, 1,
                                               Gtk.AttachOptions.FILL,
                                               Gtk.AttachOptions.SHRINK,
                                               2, 2)
            self.__table_attention_type.attach(self.__rb_attention_badge,
                                               0, 1, 1, 2,
                                               Gtk.AttachOptions.FILL,
                                               Gtk.AttachOptions.SHRINK,
                                               2, 2)

        else:
            self.__table_attention_type = Gtk.Grid()
            self.__table_attention_type.set_row_spacing(2)
            self.__table_attention_type.set_column_spacing(2)

            self.__table_attention_type.attach(self.__rb_attention_blink,
                                               0, 0, 1, 1)
            self.__table_attention_type.attach(self.__rb_attention_badge,
                                               0, 1, 1, 1)

        self.__frame_attention_type_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                          xscale=1.0, yscale=1.0)
        self.__frame_attention_type_align.set_padding(0, 0, 12, 0)
        self.__frame_attention_type_align.add(self.__table_attention_type)
        self.__frame_attention_type.add(self.__frame_attention_type_align)

        self.__frame_pdel = create_frame(_("Popup Delay(s)"))
        if build_gtk2:
            self.__sb_pdel = Gtk.SpinButton()
            self.__sb_pdel.set_adjustment(Gtk.Adjustment(1.0, 0.1, 5.0, 0.1, 1))
        else:
            self.__sb_pdel = Gtk.SpinButton.new_with_range(0.1, 5, 0.1)

        self.__sb_pdel.set_digits(1)

        self.__sb_pdel.set_update_policy(Gtk.SpinButtonUpdatePolicy.IF_VALID)
        self.__sb_pdel.set_numeric(True)
        self.__sb_pdel.set_max_length(3)
        self.__sb_pdel.set_snap_to_ticks(True)

        self.__frame_pdel_align = Gtk.Alignment(xalign=0.5, yalign=0.5,
                                                xscale=1.0, yscale=1.0)
        self.__frame_pdel_align.set_padding(0, 0, 12, 300)
        self.__frame_pdel_align.add(self.__sb_pdel)
        self.__frame_pdel.add(self.__frame_pdel_align)

        self.__misc_vbox = self.create_vbox()
        self.__misc_vbox.set_spacing(2)
        self.__misc_vbox.pack_start(self.__frame_fb_bar_col, False, False, 4)
        self.__misc_vbox.pack_start(self.__frame_attention_type, False, False, 4)
        self.__misc_vbox.pack_start(self.__frame_pdel, False, False, 4)

        self.__vbox.pack_start(self.__notebook, True, True, 4)
        self.__notebook.append_page(self.__appearance_tbl, Gtk.Label(_("Appearance")))
        self.__notebook.append_page(self.__ws_vbox, Gtk.Label(_("Workspaces")))
        self.__notebook.append_page(self.__behaviour_vbox, Gtk.Label(_("Behaviour")))
        self.__notebook.append_page(self.__panel_vbox,
                                    Gtk.Label(_("Panel Options")))
        self.__notebook.append_page(self.__misc_vbox, Gtk.Label(_("Misc")))

        self.__vbox.pack_start(Gtk.HSeparator(), True, True, 4)
        self.__vbox.pack_start(self.__hbbx, False, False, 0)

        self.set_fallback_bar_col([192, 128, 0])
        self.add(self.__vbox)
        self.show_all()

    def create_textcombo(self, items):
        """
            Convenience function to create a text combo box with a set of specified
            items

        Args:
            items : a list of the items the box is to contain

        Returns:
            a ComboBoxText
        """

        cbt = Gtk.ComboBoxText()

        # add the items
        for item in items:
            cbt.append_text(item)

        return cbt

    def set_combo_selected_item(self, cbt, item, items):
        """
            Sets a ComboBoxText's selected item

        Args:
            cbt : the ComboBoxText
            item : a string, the selected item
            items : the list of items the combo contains
        """

        index = items.index(item)
        cbt.set_active(index)

    def create_vbox(self):
        """
            Convenience function to create a Gtk2 VBox or a Gtk3 Box
            oriented vertically

        Returns:
            the vbox/box we created
        """

        if build_gtk2:
            vbox = Gtk.VBox()
        else:
            vbox = Gtk.Box()
            vbox.set_orientation(Gtk.Orientation.VERTICAL)

        return vbox

    def create_hbox(self):
        """
            Convenience function to create a Gtk2 HBox or a Gtk3 Box
            oriented horizontally

        Returns:
            the hbox/box we created
        """

        if build_gtk2:
            hbox = Gtk.HBox()
        else:
            hbox = Gtk.Box()
            hbox.set_orientation(Gtk.Orientation.HORIZONTAL)

        return hbox

    def win_delete_event(self, widget, event, data=None):
        """Callback for the preferences window delete event

        Do not delete the window, hide it instead so that it can be shown again
        later if needed

        """

        self.hide()
        return True

    def win_cancel_button_press(self, widget, event):
        """Callback for the preferences window Cancel button press

        Hide the window
        """

        self.hide()

    def draw_preview(self, drawing_area, event):
        """Draw a preview of the current appearance settings

        Draw a dark or light background to represent the panel
        Draw an appropriate type of active background
        Draw an app icon
        Draw the appropriate type of indicator (or no indicator)

        Args:

            drawing_area : our drawing area that caused the event
            event        : the event parameters
        """

        if build_gtk2:
            tgt_ctx = self.__da_preview.window.cairo_create()
        else:
            tgt_ctx = event

        # if a dark indicator is set, we draw a light background, otherwise a
        # dark background
        indicator = self.get_indicator_type()
        if indicator == IndicatorType.DARK:
            tgt_ctx.set_source_rgb(0.85, 0.85, 0.85)
        else:
            tgt_ctx.set_source_rgb(0.15, 0.21, 0.15)
        tgt_ctx.rectangle(0, 0, self.PREVIEW_SIZE * 3, self.PREVIEW_SIZE)
        tgt_ctx.fill()

        app_size = self.PREVIEW_SIZE + ind_extra_s(self.get_indicator_type())
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, app_size, self.PREVIEW_SIZE)
        ctx = cairo.Context(surface)

        # draw the active background - first we need the colour of the background in cairo
        # values
        red = self.__app.highlight_color.r / 255
        green = self.__app.highlight_color.g / 255
        blue = self.__app.highlight_color.b / 255

        bg_type = self.get_bg()
        if bg_type == IconBgType.GRADIENT:
            bgd = DefaultBackgroundDrawer(ctx, self.PREVIEW_SIZE,
                                          MatePanelApplet.AppletOrient.UP, red, green, blue)
        elif bg_type == IconBgType.ALPHAFILL:
            bgd = AlphaFillBackgroundDrawer(ctx, self.PREVIEW_SIZE,
                                            MatePanelApplet.AppletOrient.UP, red, green, blue, 0.5)
        elif bg_type == IconBgType.UNITY:
            bgd = UnityBackgroundDrawer(ctx, self.PREVIEW_SIZE,
                                        MatePanelApplet.AppletOrient.UP, red, green, blue, False, 1)
        else:
            bgd = UnityFlatBackgroundDrawer(ctx, self.PREVIEW_SIZE,
                                            MatePanelApplet.AppletOrient.UP, red, green, blue, True, 1)

        bgd.draw()

        # draw the app icon
        if bg_type in [IconBgType.UNITY, IconBgType.UNITY_FLAT]:
            pb_size = self.PREVIEW_SIZE
            pb_size = self.PREVIEW_SIZE * 3 / 4
            # create scale down pixbuf and use that
            pb_small = self.__app_pb.scale_simple(pb_size, pb_size, GdkPixbuf.InterpType.BILINEAR)

            offset = self.PREVIEW_SIZE / 2 - pb_size / 2
            Gdk.cairo_set_source_pixbuf(ctx, pb_small, offset, offset)
        else:
            offset = self.PREVIEW_SIZE / 2 - self.__app_pb.props.width / 2
            Gdk.cairo_set_source_pixbuf(ctx, self.__app_pb, offset, offset)

        ctx.paint()

        if bg_type in [IconBgType.UNITY, IconBgType.UNITY_FLAT]:
            bgd.draw_shine()

        # draw the indicator(s) if necessary
        if indicator != IndicatorType.NONE:
            # if we're showing indicators for each open window, show multiple indicators
            if self.__cb_multi_ind.get_active():
                num_ind = randint(2, 4)
            else:
                num_ind = 1

            if indicator == IndicatorType.LIGHT:
                ind = DefaultLightInd(ctx, self.PREVIEW_SIZE,
                                      MatePanelApplet.AppletOrient.UP, num_ind)
            elif indicator == IndicatorType.DARK:
                ind = DefaultDarkInd(ctx, self.PREVIEW_SIZE,
                                     MatePanelApplet.AppletOrient.UP, num_ind)
            elif indicator == IndicatorType.TBAR:
                ind = ThemeBarInd(ctx, self.PREVIEW_SIZE, MatePanelApplet.AppletOrient.UP,
                                  self.__app.applet)
            elif indicator == IndicatorType.TCIRC:
                ind = ThemeCircleInd(ctx, self.PREVIEW_SIZE, MatePanelApplet.AppletOrient.UP,
                                     self.__app.applet, num_ind)
            elif indicator == IndicatorType.TSQUARE:
                ind = ThemeSquareInd(ctx, self.PREVIEW_SIZE, MatePanelApplet.AppletOrient.UP,
                                     self.__app.applet, num_ind)
            elif indicator == IndicatorType.TTRI:
                ind = ThemeTriInd(ctx, self.PREVIEW_SIZE, MatePanelApplet.AppletOrient.UP,
                                  self.__app.applet, num_ind)
            elif indicator == IndicatorType.TDIA:
                ind = ThemeDiaInd(ctx, self.PREVIEW_SIZE, MatePanelApplet.AppletOrient.UP,
                                  self.__app.applet, num_ind)
            elif indicator == IndicatorType.SUBWAY:
                active = bool(randint(0, 1))
                ind = SubwayInd(ctx, self.PREVIEW_SIZE, MatePanelApplet.AppletOrient.UP,
                                self.__app.applet, num_ind, surface, active)
            else:
                ind = None

            if ind is not None:
                ind.draw()

            # now draw to the screen
            if build_gtk2:
                tgt_ctx = drawing_area.window.cairo_create()
                tgt_ctx.rectangle(event.area.x, event.area.y,
                                  event.area.width, event.area.height)
                tgt_ctx.clip()

                tgt_ctx.set_source_surface(surface, self.PREVIEW_SIZE, 0)

                tgt_ctx.paint()
                tgt_ctx = None
            else:
                tgt_ctx.set_source_surface(surface, self.PREVIEW_SIZE, 0)
                tgt_ctx.paint()

            ctx = None

    def color_change_toggled(self, widget):
        """Handler for the panel color change checkbox toggled event

        If the panel colour change option is selected, enable the checkbox that
        specifies whether or not all panels are to change colour

        """

        self.__cb_dock_panel_only.set_sensitive(self.__cb_panel_color_change.get_active())

    def theme_changed(self, widget):
        """ Handler for the theme being changed

        Set the appropriate indicator and background type for the theme
        """

        theme = self.get_theme()

        # enable or disable the indicator and backgroud type combos depending
        # on whether or not we are using a custom themes
        self.__cbt_ind_type.set_sensitive(theme == ThemeType.CUSTOM)
        self.__cbt_icon_bg.set_sensitive(theme == ThemeType.CUSTOM)

        if theme == ThemeType.DEFAULT:
            self.set_indicator(IndicatorType.LIGHT)
            self.set_bg(IconBgType.GRADIENT)
        elif theme == ThemeType.UNITY:
            self.set_indicator(IndicatorType.TTRI)
            self.set_bg(IconBgType.UNITY)
        elif theme == ThemeType.UNITY_FLAT:
            self.set_indicator(IndicatorType.TTRI)
            self.set_bg(IconBgType.UNITY_FLAT)
        elif theme == ThemeType.SUBWAY:
            self.set_indicator(IndicatorType.SUBWAY)
            self.set_bg(IconBgType.ALPHAFILL)

        # if a custom


    def setting_toggled(self, widget):
        """ Handler for updating the preview when appearance settings are changed

        Args:
            widget: the widget the caused the event
        """
        self.__da_preview.queue_draw()

    def rb_no_ind_toggled(self, widget):
        """ Handler for the no indicator radio button toggled event, also
            used by the bar indicator rb

        If either the no indicator or bar indicator ption is selected, disable
        the multiple indicator checkbox

        Update the appearance preview
        """

        self.__cb_multi_ind.set_sensitive(self.__rb_no_ind.get_active() is not True and
                                          self.__rb_bar_ind.get_active() is not True)
        self.setting_toggled(widget)

    def set_dock_size_visible(self, vis):
        """ Set whether the dock size frame is visible or not

        Params: vis - bool

        """

        self.__frame_dock_size.set_visible(vis)

    def get_indicator_type(self):
        """Get the indicator type specified in the preferences window.

        Returns : IndicatorType
        """

        indicator = self.__cbt_ind_type.get_active_text()
        if indicator == _("Default light"):
            return IndicatorType.LIGHT
        elif indicator == _("Default dark"):
            return IndicatorType.DARK
        elif indicator == _("Single bar"):
            return IndicatorType.TBAR
        elif indicator == _("Circle"):
            return IndicatorType.TCIRC
        elif indicator == _("Square"):
            return IndicatorType.TSQUARE
        elif indicator == _("Triangle"):
            return IndicatorType.TTRI
        elif indicator == _("Diamond"):
            return IndicatorType.TDIA
        elif indicator == _("Subway"):
            return IndicatorType.SUBWAY
        else:
            return IndicatorType.NONE

    def set_indicator(self, indicator):
        """Set the indicator type

        Args : indicator - an IndicatorType
        """

        if indicator == IndicatorType.LIGHT:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Default light"),
                                         self.__indicator_options)
        elif indicator == IndicatorType.DARK:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Default dark"),
                                         self.__indicator_options)
        elif indicator == IndicatorType.TBAR:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Single bar"),
                                         self.__indicator_options)
        elif indicator == IndicatorType.TCIRC:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Circle"),
                                         self.__indicator_options)
        elif indicator == IndicatorType.TSQUARE:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Square"),
                                         self.__indicator_options)
        elif indicator == IndicatorType.TTRI:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Triangle"),
                                         self.__indicator_options)
        elif indicator == IndicatorType.TDIA:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Diamond"),
                                         self.__indicator_options)
        elif indicator == IndicatorType.SUBWAY:
            self.set_combo_selected_item(self.__cbt_ind_type, _("Subway"),
                                         self.__indicator_options)
        else:
            self.set_combo_selected_item(self.__cbt_ind_type, _("None"),
                                         self.__indicator_options)

    def get_multi_ind(self):
        """Gets whether or not to use display an indicator for each open
           window that a docked app has

        Returns: boolean
        """

        return self.__cb_multi_ind.get_active()

    def set_bg(self, bg):
        """ Set the active icon background type

        Args : bg - an IconBgType
        """

        if bg == IconBgType.GRADIENT:
            self.set_combo_selected_item(self.__cbt_icon_bg, _("Gradient fill"), self.__bg_options)
        elif bg == IconBgType.ALPHAFILL:
            self.set_combo_selected_item(self.__cbt_icon_bg, _("Solid fill"), self.__bg_options)
        elif bg == IconBgType.UNITY:
            self.set_combo_selected_item(self.__cbt_icon_bg, _("Unity"), self.__bg_options)
        else:
            self.set_combo_selected_item(self.__cbt_icon_bg, _("Unity Flat"), self.__bg_options)

    def get_bg(self):
        """ Gets the currently selected active icon background type

        Returns: An IconBgType

        """

        bg = self.__cbt_icon_bg.get_active_text()
        if bg == _("Gradient fill"):
            return IconBgType.GRADIENT
        elif bg == _("Solid fill"):
            return IconBgType.ALPHAFILL
        elif bg == _("Unity"):
            return IconBgType.UNITY
        else:
            return IconBgType.UNITY_FLAT

    def set_theme(self, theme):
        """ Sets the theme

        Args:
            theme : a ThemeType
        """

        if theme == ThemeType.DEFAULT:
            self.set_combo_selected_item(self.__cbt_theme, _("Default"), self.__theme_options)
        elif theme == ThemeType.UNITY:
            self.set_combo_selected_item(self.__cbt_theme, _("Unity"), self.__theme_options)
        elif theme == ThemeType.UNITY_FLAT:
            self.set_combo_selected_item(self.__cbt_theme, _("Unity Flat"), self.__theme_options)
        elif theme == ThemeType.SUBWAY:
            self.set_combo_selected_item(self.__cbt_theme, _("Subway"), self.__theme_options)
        elif theme == ThemeType.CUSTOM:
            self.set_combo_selected_item(self.__cbt_theme, _("Custom"), self.__theme_options)

    def get_theme(self):
        """ Get the currently selected theme

        Returns : a ThemeType
        """

        theme = self.__cbt_theme.get_active_text()
        if theme == _("Default"):
            return ThemeType.DEFAULT
        elif theme == _("Unity"):
            return ThemeType.UNITY
        elif theme == _("Unity Flat"):
            return ThemeType.UNITY_FLAT
        elif theme == _("Subway"):
            return ThemeType.SUBWAY
        else:
            return ThemeType.CUSTOM

    def set_multi_ind(self, use_multi_ind):
        """Sets whether or not to display multiple indicators

        Args: use_multi_ind - boolean
        """

        self.__cb_multi_ind.set_active(use_multi_ind)

    def get_show_unpinned_apps_on_all_ws(self):
        """Gets whether unpinned apps are displayed in the dock on all workspaces

        Returns: boolean
        """

        return self.__rb_unpinned_all_ws.get_active()

    def set_show_unpinned_apps_on_all_ws(self, show_on_all):
        """Sets whether unpinned apps are displayed in the dock on all workspaces

        Args: show_on_all - boolean
        """

        if show_on_all:
            self.__rb_unpinned_all_ws.set_active(True)
        else:
            self.__rb_unpinned_cur_ws.set_active(True)

    def get_show_pinned_apps_on_all_ws(self):
        """Gets whether pinned apps are displayed in the dock on all workspaces
           or just on the workspace where they were created

        Returns: boolean
        """

        return self.__rb_pinned_all_ws.get_active()

    def set_show_pinned_apps_on_all_ws(self, show_on_all):
        """Sets whether pinned apps are displayed in the dock on all workspaces

        Args: show_on_all - boolean
        """

        if show_on_all:
            self.__rb_pinned_all_ws.set_active(True)
        else:
            self.__rb_pinned_pin_ws.set_active(True)

    def get_pan_act(self):
        """ Gets whether or not the show the action list on the panel
            right click menu, rather than as a popup

            Returns: boolean
        """

        return self.__cb_pan_act.get_active()

    def set_pan_act(self, pan_act):
        """ Sets whether or not the show the action list on the panel
            right click menu, rather than as a popup

            Args:
                pan_act: boolean
        """

        self.__cb_pan_act.set_active(pan_act)

    def get_use_win_list(self):
        """Gets whether to use the dock's window list to select windows
        or whether to use Compiz thumbnail previews

            Returns: boolean
        """

        return self.__rb_win_list.get_active()

    def set_use_win_list(self, use_win_list):
        """Sets whether to use the dock's window list to select windows
        or whether to use Compiz thumbnail previews

        Args: use_win_list - boolean
        """

        if use_win_list:
            self.__rb_win_list.set_active(True)
        else:
            self.__rb_win_thumb.set_active(True)

    def get_click_action(self):
        """ Gets the action specified for when a running app's dock icon
            is clicked

        Returns : a ClickActionType value

        """

        if self.__rb_win_list.get_active():
            return ClickActionType.WIN_LIST
        elif self.__rb_win_thumb.get_active():
            return ClickActionType.COMPIZ
        else:
            return ClickActionType.MINMAX

    def set_click_action(self, click_action):
        """ Sets the action specified for when a running app's dock icon
            is clicked

        Args:
            click_action: A ClickActionType value
        """

        if click_action == ClickActionType.WIN_LIST:
            self.__rb_win_list.set_active(True)
        elif click_action == ClickActionType.COMPIZ:
            self.__rb_win_thumb.set_active(True)
        else:
            self.__rb_win_minmax.set_active(True)

    def get_change_panel_color(self):
        """ Get whether the panel colour is to be changed according to the
            current wallpaper

        Returns: boolean
        """

        return self.__cb_panel_color_change.get_active()

    def set_change_panel_color(self, change_color):
        """ Sets whether the panel color is to be changed according to the
            current wallpaper

        Args: change_color - boolean
        """

        self.__cb_panel_color_change.set_active(change_color)

    def get_change_dock_color_only(self):
        """ Get whether only the panel containing the dock is to be changed
            when setting the panel colour according to the current wallpaper

        Returns: boolean
        """

        return self.__cb_dock_panel_only.get_active()

    def rb_variable_ds_toggled(self, widget):
        """ Handler for dock fixed size setting changes

        Update the ui according to the current settingd

        """

        size_fixed = self.__rb_fixed_ds.get_active()

        # enable/diable the parts of the interface that allow the user to set the number
        # of app icons in a fixed size dock
        self.__lbl_fixed_size.set_sensitive(size_fixed)
        self.__sb_fixed_size.set_sensitive(size_fixed)
        self.__lbl_fixed_size1.set_sensitive(size_fixed)

    def set_fixed_size(self, fixed_size, num_icons, mutiny_layout):
        """ Set whether or not the dock is a fixed size, and if so the number
            of app icons it can contain before enabling scrolling

        If we're using the Mutiny panel layout, disable all fixed size settings

        Params:
            fixed_size : bool - whether or not the dock is a fixed size
            num_icons  : int - the number of icons
            mutiny_layout : bool - indicates whether or not we're using the Mutiny layour
        """

        self.__rb_variable_ds.connect("toggled", self.rb_variable_ds_toggled)

        if not mutiny_layout:
            self.__rb_fixed_ds.set_sensitive(True)
            self.__rb_variable_ds.set_sensitive(True)
            self.__rb_fixed_ds.set_active(fixed_size)
            self.rb_variable_ds_toggled(self.__rb_variable_ds)  # ensure ui is updated
            self.__sb_fixed_size.set_value(num_icons)

        else:
            self.__lbl_fixed_size.set_sensitive(False)
            self.__sb_fixed_size.set_sensitive(False)
            self.__lbl_fixed_size1.set_sensitive(False)
            self.__rb_fixed_ds.set_sensitive(False)
            self.__rb_variable_ds.set_sensitive(False)
            self.__rb_variable_ds.disconnect_by_func(self.rb_variable_ds_toggled)
            self.__rb_fixed_ds.set_active(fixed_size)

    def get_fixed_size(self):
        """ Get the dock fixed size settings

        Note: the dock should not call this method if the Mutiny panel layout is being
              used

        Returns:
            a bool - whether or not the dock is of fixed size
            an int - the number of app icons in a fixed size dock
        """

        return self.__rb_fixed_ds.get_active(), self.__sb_fixed_size.get_value()

    def set_change_dock_color_only(self, dock_only):
        """ Sets whether only the panel containing the dock is to be changed
            when settings the panel colour according to the current wallpaper

        Args: dock_only - boolean
        """

        self.__cb_dock_panel_only.set_active(dock_only)

    def get_win_cur_ws_only(self):
        """ Gets whether the dock will show indicators/window list items for
            the current workspace only

        Returns: boolean
        """

        return self.__cb_win_cur_ws.get_active()

    def set_win_cur_ws_only(self, win_cur_ws_only):
        """ Sets whether the dock will show indicators/window list items for
            the current workspace only

        Args: win_cur_ws_only - boolean
        """

        self.__cb_win_cur_ws.set_active(win_cur_ws_only)

    def get_win_switch_unity_style(self):
        """ Gets whether the dock will focus the last active window of an app
            on first click

        Returns: boolean
        """

        return self.__cb_win_switch_unity_style.get_active()

    def set_win_switch_unity_style(self, win_switch_unity_style):
        """ Sets whether the dock will focus the last active window of an app
            on first click

        Args: win_switch_unity_style - boolean
        """

        self.__cb_win_switch_unity_style.set_active(win_switch_unity_style)

    def set_fallback_bar_col(self, colrgb):
        """
            Set the colour of the fallback bar indicator colour button

        Args:
            colrgb : a list containing the r,g and b colour components(0-255) as strings
        """

        colstr = "#%0.2X%0.2X%0.2X" % (int(colrgb[0]), int(colrgb[1]), int(colrgb[2]))

        if build_gtk2:
            cbrgba = Gdk.color_parse(colstr)
            self.__cbtn_fb_bar_col.set_color(color=cbrgba)
        else:
            cbrgba = Gdk.RGBA()
            cbrgba.parse(colstr)
            self.__cbtn_fb_bar_col.set_rgba(cbrgba)

        self.__cbtn_fb_bar_col.set_use_alpha(False)

    def get_fallback_bar_col(self):
        """
            Get the colour of the fallback bar indicator colour button

        Returns:
            a list containing the r, g, and b colour components(0-255)
        """

        if build_gtk2:
            cbrgba = self.__cbtn_fb_bar_col.get_color()
        else:
            cbrgba = self.__cbtn_fb_bar_col.get_rgba().to_color()

        return ["%s" % int(cbrgba.red / 256), "%s" % int(cbrgba.green / 256), "%s" % int(cbrgba.blue / 256)]

    def set_app_spacing(self, spacing):
        """ Set the amount of space between icons in the dock
        """

        self.__sb_spc.set_value(spacing)

    def get_app_spacing(self):
        """
            Get the amount of space between icons in the dock

        :return: int

        """

        self.__sb_spc.update()
        return self.__sb_spc.get_value()

    def set_attention_type(self, attention_type):
        """ Set the attention type

        Args : attention_type: An AttentionType e.g. AttentionType.BLINK
        """
        if attention_type == AttentionType.BLINK:
            self.__rb_attention_blink.set_active(True)
        else:
            self.__rb_attention_badge.set_active(True)

    def get_attention_type(self):
        """
            Get the attention type

        Returns: An AttentionType

        """

        if self.__rb_attention_blink.get_active():
            return AttentionType.BLINK
        else:
            return AttentionType.SHOW_BADGE

    def set_popup_delay(self, delay):
        """ Set the amount of space between icons in the dock

        Args:
            delay: the delay in ms
        """

        # convert delay to seconds
        self.__sb_pdel.set_value(delay / 1000)

    def get_popup_delay(self):
        """
            Get the popup delay, converting from seconds to ms

        :return: int

        """

        self.__sb_pdel.update()
        return int(self.__sb_pdel.get_value() * 1000)


def main():
    """main function - debug code can go here"""
    # dpw = DockPrefsWindow(Gtk.main_quit)
    # Gtk.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
    Provide a base class for the dock's popup windows.

    Such a window will function in a similar way to a tooltip i.e.
    it will appear when the mouse hovers over a dock icon and
    will disappear if the mouse moves away from the window or
    the dock applet.

    The window's foreground/background colours will be set from the current
    theme or if the dock applet is setting the panel colour, the panel colours

    The will use a grid/table to display a border around the window contents,
    and descendant classes will need to create and set the window's
    main widget/container

"""

#
# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure

build_gtk2 = False

import gi
if build_gtk2:
    gi.require_version("Gtk", "2.0")
    gi.require_version("Wnck", "1.0")
else:
    gi.require_version("Gtk", "3.0")
    gi.require_version("Wnck", "3.0")

gi.require_version("MatePanelApplet", "4.0")

from gi.repository import Gtk
from gi.repository import GdkPixbuf
from gi.repository import Gio
from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import MatePanelApplet

import cairo
from time import sleep

import docked_app
from math import pi


CONST_TIMER_DELAY = 1000
CONST_ICON_SIZE = 16


class DockPopup(Gtk.Window):
    """

        Attributes : __mouse_areas : a list containing Gdk.Rectangle objects -
                     used when the window has been shown and defines the on
                     screen areas in which the mouse pointer must stay,
                     otherwise the window list will be hidden. The rectangles
                     should therefore include the applet, the area between
                     the window and the applet, and the window itself with a
                     suitable buffer area around it
            __timer_id : a ref to a timer used for periodically checking the
                         mouse cursor position to see if it is within the areas
                         specified in __mouse_areas
            __the_app : the docked_app to which the window list relates
            __icontheme : used for drawing app icons in the popup. This
                          is set from the Gtk.Icon used by the dock, and
                          will therefore track changes to the icon theme whilst
                          the dock is running
            __icon_size : the size in pixels at which app icons will be drawn

            __win_w : the width of the window
            __win_h : the height of the window
            __bgr, __bgg, __bgb : the r,g,b panel colour components (0-255)
            __fgr, __fgg, __fgb : the r,g,b foreground colour components
            __hlr, __hlg, __hlb : the r,g,b highlight colour components

            The attributes below are used for positioning this window relative
            to the applet and it's panel:
            __app_x : the x position of the docked app in root coordinates
            __app_y : the y position of the docked app in root coordinates
            __applet_x : the x position of the applet in root coordinates
            __applet_y : the y position of the applet in root coordinates
            __applet_w : the width of the applet in pixels
            __applet_h : the height of the applet in pixels
            __panel_orient : the orienation of the MATE panel the applet is on

            __do_window_shaping : whether or not the window can be shaped,
                                  e.g. have rounded corners. Depends on
                                  Gtk3 and gi module >= 3.26.0
    """

    def __init__(self, wnck_screen, panel_orient, scroll_adj):
        """
        create the window and its contents

        Args:
            wnck_screen: the wnck_screen of the applet
            panel_orient : the orientation of the panel
            scroll_adj   : an adjustment to be applied to the window position
                           because the dock has scrolling enabled
        """

        def create_drawing_area(width, height, draw_event):
            # convenience func to create a drawing area with a specified
            # width, height and draw event
            da = Gtk.DrawingArea()
            da.set_size_request(width, height)
            if build_gtk2:
                da.connect("expose-event", draw_event)
            else:
                da.connect("draw", draw_event)

            return da

        super().__init__(title="")
        self.wnck_screen = wnck_screen
        self.set_decorated(False)  # we don't want a titlebar..
        self.set_skip_taskbar_hint(True)  # we don't want to be in the taskbar
        self.set_accept_focus(False)

        self.set_keep_above(True)

        self.__scroll_adj = scroll_adj
        self.__icontheme = None
        self.__icon_size = 16  # small default icon size
        self.__timer_id = None
        self.__dismissed = False

        self.__the_app = None
        self.__app_pb = None

        self.__win_w = 0
        self.__win_h = 0

        self.__app_x = 0
        self.__app_y = 0
        self.__panel_orient = panel_orient

        self.__bgr = 0
        self.__bgg = 0
        self.__bgb = 0
        self.__fgr = 0
        self.__fgg = 0
        self.__fgb = 0
        self.__hlr = 0
        self.__hlg = 0
        self.__hlb = 0
        self.__applet_x = 0
        self.__applet_y = 0
        self.__applet_w = 0
        self.__applet_y = 0
        self.__applet_h = 0

        # create ui
        if build_gtk2:
            self.__grid = Gtk.VBox()
            self.__grid.set_spacing(0)
            self.__grid = Gtk.Table(rows=3, columns=3)
            self.__grid.set_row_spacings(0)
            self.__grid.set_col_spacings(0)
        else:
            self.__grid = Gtk.Grid()
            self.__grid.set_orientation(Gtk.Orientation.VERTICAL)
            self.__grid.hexpand = True
            self.__grid.vexpand = True
            self.hexpand = True
            self.vexpand = True

        # set vars used when drawing the window border
        self.__border_width = 15
        self.__border_line = 4
        self.__line_width = 2
        self.__line_curve = 5.0
        self.__pointer_size = 16

        # add drawing areas to all outsides of the 3x3 grid
        # if we're showing shaped windows then the drawing area nearest the panel
        # needs to be expanded so that that portion of the window it can be shaped
        #  into a pointer to the app icon

        if build_gtk2:
            self.__do_window_shaping = False
        else:
            gi_ver = GObject.pygobject_version
            self.__do_window_shaping = gi_ver[0] > 3 or ((gi_ver[0] == 3) and (gi_ver[1] >= 26))

        da_height = self.__border_width
        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.DOWN:
            da_height += self.__pointer_size
        self.__da_top = create_drawing_area(self.__border_width,
                                            da_height,
                                            self.draw_top_border)

        da_width = self.__border_width
        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
            da_width += self.__pointer_size
        self.__da_left = create_drawing_area(da_width,
                                             self.__border_width,
                                             self.draw_left_border)

        da_width = self.__border_width
        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.LEFT:
            da_width += self.__pointer_size
        self.__da_right = create_drawing_area(da_width,
                                              self.__border_width,
                                              self.draw_right_border)

        da_height = self.__border_width
        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.UP:
            da_height += self.__pointer_size
        self.__da_bottom = create_drawing_area(self.__border_width,
                                               da_height,
                                               self.draw_bottom_border)

        if build_gtk2:
            self.__grid.attach(self.__da_top, 0, 3, 0, 1, xpadding=0,
                               ypadding=0)
            self.__grid.attach(self.__da_left, 0, 1, 1, 2)
            self.__grid.attach(self.__da_right, 2, 3, 1, 2)
            self.__grid.attach(self.__da_bottom, 0, 3, 2, 3)
        else:
            self.__grid.attach(self.__da_top, 0, 0, 3, 1)
            self.__grid.attach(self.__da_left, 0, 1, 1, 1)
            self.__grid.attach(self.__da_right, 2, 1, 1, 1)
            self.__grid.attach(self.__da_bottom, 0, 2, 3, 1)

        self.add(self.__grid)

        self.__mouse_areas = []

        # connect handlers for the show and hide events
        self.connect("show", self.win_shown)
        self.connect("hide", self.win_hidden)

        self.connect("configure-event", self.win_configure)
        self.connect("size-allocate", self.size_allocate)

    def set_main_widget(self, widget):
        """ Attaches the main component (a widget or container) to the center
            position of the grid

        Args:
            widget : the widget or container to add

        """
        if build_gtk2:
            self.__grid.attach(widget, 1, 2, 1, 2)
        else:
            self.__grid.attach(widget, 1, 1, 1, 1)

    def set_colours(self, panel_colour):
        """ Sets the window background, foreground and highlight colours
            to default values

        The background colour will match the panel containing the applet.

        If a custom colour is set for the panel, use that for the background
        and set the foreground colour to be either full white or black
        (depending on the background colour). The highlight colour will Also
        be set depending on the background colour.

        For Gtk3, if the panel is set to use the theme's colours, the
        background, foreground and  highilight colours will all be set from
        the current theme

        For Gtk2, where we can't access the styles associated with the current
        theme because of introspection errors, set the background to black,
        foreground to white and the highlight colour to a dark grey


        Args:
            panel_colour : If a custom panel colour has been set, this will
                           be a tuple of 3 x int - the r, g, b colour
                           components. Otherwise it will be None

        """

        if panel_colour is None:
            if build_gtk2:
                self.__bgr = self.__bgg = self.__bgb = 0
                self.__fgr = self.__fgg = self.__fgb = 255
                self.__hlr = self.__hlg = self.__hlb = 64
            else:

                context = self.get_style_context()
                state = Gtk.StateType.NORMAL
                # we want the colors for the MATE panel (preferably), or the
                # Gnome menu bar
                # context.add_class("gnome-panel-menu-bar")
                # context.add_class("mate-panel-menu-bar")

                # background
                c_info = context.lookup_color("dark_bg_color")
                if c_info[0]:
                    bgcol = c_info[1]
                    self.__bgr = int(bgcol.red * 255)
                    self.__bgg = int(bgcol.green * 255)
                    self.__bgb = int(bgcol.blue * 255)

                c_info = context.lookup_color("dark_fg_color")
                if c_info[0]:
                    fcol = c_info[1]
                    self.__fgr = int(fcol.red * 255)
                    self.__fgg = int(fcol.green * 255)
                    self.__fgb = int(fcol.blue * 255)

                sel_bg = context.lookup_color("theme_selected_bg_color")
                if sel_bg[0]:
                    hcol = sel_bg[1]
                    self.__hlr = int(hcol.red * 255)
                    self.__hlg = int(hcol.green * 255)
                    self.__hlb = int(hcol.blue * 255)
                else:
                    # assume what is hopefully a decent looking highlight
                    # colour
                    self.__hlr = (self.__bgr + 64) % 256
                    self.__hlg = (self.__bgg + 64) % 256
                    self.__hlb = (self.__bgb + 64) % 256
        else:
            # custom panel colour...
            self.__bgr = panel_colour[0]
            self.__bgg = panel_colour[1]
            self.__bgb = panel_colour[2]

            # set foreground colour according to the background colour
            # 384 equates to average rgb values of 128 per colour component and
            # therefore represents a mid value
            if (self.__bgr + self.__bgg + self.__bgb) > 384:

                self.__fgr = self.__fgg = self.__fgb = 0  # dark fg colour
            else:
                self.__fgr = self.__fgg = self.__fgb = 255  # light fg color

            # highlight colour
            self.__hlr = (self.__bgr + 64) % 256
            self.__hlg = (self.__bgg + 64) % 256
            self.__hlb = (self.__bgb + 64) % 256

    def set_bg_col(self, bgr, bgg, bgb):
        """ Sets the background colour of the window

        Also, set a foreground colour that will contrast with the background
        colour (so we can read text etc...)

        Args:
            bgr, bgg, bgb : the background rgb colour components

        """

        self.__bgr = bgr
        self.__bgg = bgg
        self.__bgb = bgb

        # set foreground colour according to the background colour
        if (bgr + bgg + bgb) > 384:     # 384 equates to average rgb values of 128
                                        # per colour component and therefore
                                        # represents a mid value
            self.__fgr = self.__fgg = self.__fgb = 0  # dark fg colour
        else:
            self.__fgr = self.__fgg = self.__fgb = 255  # light fg color

    def set_fg_col(self, fgr, fgg, fgb):
        """
        Put some stuff here...
        """

        self.__fgr = fgr
        self.__fgg = fgg
        self.__fgb = fgb

    def win_shown(self, widget):
        """ Event handler for the window's show event

            Get the window's size so that its position can be set and mouse
            areas created
        """

        if build_gtk2:
            self.set_win_position()
        else:
            if (self.__win_w == 0) or (self.__win_h == 0):
                self.__win_w, self.__win_h = self.get_size()

        self.start_mouse_area_timer()

    def set_win_position(self):
        """
            Move the window so that it appears near the panel and centered on
            the app (has to be done here for Gtk3 reasons)

            Create mouse areas as required so we can check when the mouse
            leaves the window

            Instantiate a timer to periodically check the mouse cursor position

        """

        def create_rect(x, y, w, h):
            """ Convenience function to create and return a Gdk.Rectangle
                (needed with Gtk3)
            """

            if build_gtk2:
                rect = Gdk.Rectangle(0, 0, 0, 0)
            else:
                rect = Gdk.Rectangle()

            rect.x = x
            rect.y = y
            rect.width = w
            rect.height = h

            return rect

        # set how many pixels away from the panel the window list will appear
        if self.__do_window_shaping:
            panel_space = 5
        else:
            panel_space = 10

        # size of the border (in pixels) around the window
        # list where the mouse must remain, outside of which
        # the window list will hide
        win_border = 15

        screen = self.get_screen()

        # get the monitor that the applet is on
        # Note: we can't rely on the panel's dconf settings for this
        # as the monitor setting there doesn't seem to work reliably
        monitor = screen.get_monitor_at_point(self.__applet_x, self.__applet_y)
        if build_gtk2:
            mon_geom = create_rect(0, 0, 0, 0)
            screen.get_monitor_geometry(monitor, mon_geom)
        else:
            mon_geom = screen.get_monitor_geometry(monitor)

        # if the size of the window hasnt been set (because the configure-event
        # doesn't always fire if the window list is empty) use an alternative
        # method to get the window width and height
        # work out where to place the window - adjacent to the panel and
        #  centered on the highlighted dock app and add appropriate mouse areas

        # first, a mouse area to cover the entire applet
        self.add_mouse_area(create_rect(self.__applet_x, self.__applet_y,
                                        self.__applet_w, self.__applet_h))

        app_alloc = self.__the_app.drawing_area.get_allocation()

        if self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
            centre_pos = self.__app_y + (app_alloc.height / 2) - self.__scroll_adj
            win_x = self.__applet_x + self.__applet_w + panel_space
            win_y = centre_pos - (self.__win_h / 2)

            # adjust win_y in case we're off the top the screen, or the
            # monitor ...
            if win_y < mon_geom.y + panel_space:
                win_y = panel_space

            # adjust win_y if case the window list extends beyound the end of
            # the panel ..
            if (win_y + self.__win_h) > mon_geom.y + mon_geom.height:
                win_y = mon_geom.y + mon_geom.height - self.__win_h - panel_space

            # setup a new mouse area covering the window (minus a border) and
            # extending to the panel
            self.add_mouse_area(create_rect(self.__applet_x,
                                            win_y - win_border,
                                            win_x + self.__win_w + win_border,
                                            self.__win_h + (2 * win_border)))

        elif self.__panel_orient == MatePanelApplet.AppletOrient.LEFT:
            centre_pos = self.__app_y + (app_alloc.height / 2) - self.__scroll_adj
            win_x = self.__applet_x - panel_space - self.__win_w
            win_y = centre_pos - (self.__win_h / 2)

            # adjust win_y in case we're off the top the screen...
            if win_y < mon_geom.y + panel_space:
                win_y = mon_geom.y + panel_space

            # adjust win_y if case the window list extends beyound the end of
            # the panel ..
            if (win_y + self.__win_h) > mon_geom.y + mon_geom.height:
                win_y = mon_geom.y + mon_geom.height - self.__win_h - panel_space

            # setup a new mouse area covering the window (minus a border) and
            # extending to the panel
            self.add_mouse_area(create_rect(win_x - win_border,
                                            win_y - win_border,
                                            (self.__win_w + win_border +
                                             panel_space + app_alloc.width),
                                            self.__win_h + (2 * win_border)))

        elif self.__panel_orient == MatePanelApplet.AppletOrient.DOWN:
            centre_pos = (self.__app_x + app_alloc.width / 2) - self.__scroll_adj
            win_x = centre_pos - (self.__win_w / 2)
            win_y = self.__applet_y + self.__applet_h + panel_space

            # adjust win_x in case we're off the left of the screen...
            if win_x < mon_geom.x + panel_space:
                win_x = mon_geom.x + panel_space

            # adjust win_x if case the window list extends beyond the end of
            # the panel ..
            if (win_x + self.__win_w) > mon_geom.x + mon_geom.width:
                win_x = mon_geom.x + mon_geom.width - self.__win_w - panel_space

            # setup a new mouse area covering the window (minus a border) and
            # extending to the panel
            self.add_mouse_area(create_rect(win_x - win_border,
                                            self.__applet_y,
                                            self.__win_w + (2 * win_border),
                                            win_y + self.__win_h + win_border))
        else:
            centre_pos = (self.__app_x + app_alloc.width / 2) - self.__scroll_adj
            win_x = centre_pos - (self.__win_w / 2)
            win_y = self.__applet_y - panel_space - self.__win_h

            # adjust win_x in case we're off the left of the screen...
            if win_x < mon_geom.x + panel_space:
                win_x = mon_geom.x + panel_space

            # adjust win_x if case the window list extends beyond the end of
            # the panel ..
            if (win_x + self.__win_w) > mon_geom.x + mon_geom.width:
                win_x = mon_geom.x + mon_geom.width - self.__win_w - panel_space

            # setup a new mouse area covering the window (minus a border) and
            # extendingto the panel
            self.add_mouse_area(create_rect(win_x - win_border,
                                            win_y - win_border,
                                            self.__win_w + (2 * win_border),
                                            self.__win_h + win_border +
                                            panel_space + app_alloc.height))

        self.move(win_x, win_y)

    def start_mouse_area_timer(self):
        """ Start the timer that that monitors the mouse position
        """

        # remove any old timer...
        self.stop_mouse_area_timer()

        self.__timer_id = GObject.timeout_add(CONST_TIMER_DELAY, self.do_timer)

    def stop_mouse_area_timer(self):
        """ Stop the timer that monitors the mouse position
        """
        #
        if self.__timer_id is not None:
            GObject.source_remove(self.__timer_id)
            self.__timer_id = None

    def win_configure(self, widget, event):
        """ Event handler for the window's configure event

        Stores the new width and height of the window

        Args:
            widget : the widget that caused the event (i.e. self)
            event  : the event parameters
        """

        # if the new size of the window isn't the same as the old one, we need
        # to recaclulate the window position and mouse areas

        return

    def size_allocate(self, widget, event):

        def draw_rounded(cr, area, radius):
            """ draws rectangles with rounded (circular arc) corners """
            # Attribution: https://gist.github.com/kamiller/3013605

            a, b, c, d = area
            cr.arc(a + radius, c + radius, radius, 2 * (pi / 2), 3 * (pi / 2))
            cr.arc(b - radius, c + radius, radius, 3 * (pi / 2), 4 * (pi / 2))
            cr.arc(b - radius, d - radius, radius, 0 * (pi / 2), 1 * (pi / 2))  # ;o)
            cr.arc(a + radius, d - radius, radius, 1 * (pi / 2), 2 * (pi / 2))
            cr.close_path()

            cr.stroke_preserve()

        if (event.width != self.__win_w) or (event.height != self.__win_h):
            self.__win_w = event.width
            self.__win_h = event.height

            self.set_win_position()

        if not self.__do_window_shaping:
            return

        # round the corners of the portion of the window containing the widget and border
        allocation = self.get_allocation()

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                     allocation.width,
                                     allocation.height)

        ctx = cairo.Context(surface)

        if self.__panel_orient == MatePanelApplet.AppletOrient.UP:
            draw_rounded(ctx, [allocation.x, allocation.width,
                               allocation.y, allocation.height - self.__pointer_size], 10)
        elif self.__panel_orient == MatePanelApplet.AppletOrient.DOWN:
            draw_rounded(ctx, [allocation.x, allocation.width,
                               allocation.y + self.__pointer_size,
                               allocation.height], 10)
        elif self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
            draw_rounded(ctx, [self.__pointer_size + 1, allocation.width,
                               allocation.y, allocation.height], 10)
        else:
            draw_rounded(ctx, [allocation.x, allocation.width - self.__pointer_size,
                               allocation.y, allocation.height], 10)

        ctx.set_source_rgba(1, 1, 1, 1)

        ctx.fill()

        # now create a pointer to the app's icon in the dock
        if self.__panel_orient == MatePanelApplet.AppletOrient.UP:
            ctx.move_to(allocation.width / 2 - self.__pointer_size,
                        allocation.height - self.__pointer_size)
            ctx.line_to(allocation.width / 2, allocation.height)
            ctx.line_to(allocation.width / 2 + self.__pointer_size,
                        allocation.height - self.__pointer_size)
        elif self.__panel_orient == MatePanelApplet.AppletOrient.DOWN:
            ctx.move_to(allocation.width / 2 - self.__pointer_size,
                        self.__pointer_size)
            ctx.line_to(allocation.width / 2, 0)
            ctx.line_to(allocation.width / 2 + self.__pointer_size,
                        self.__pointer_size)
        elif self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
            ctx.move_to(self.__pointer_size,
                        allocation.height / 2 - self.__pointer_size)
            ctx.line_to(0, allocation.height / 2)
            ctx.line_to(self.__pointer_size,
                        allocation.height / 2 + self.__pointer_size)
        else:
            ctx.move_to(allocation.width - self.__pointer_size,
                        allocation.height / 2 - self.__pointer_size)
            ctx.line_to(allocation.width, allocation.height / 2)
            ctx.line_to(allocation.width - self.__pointer_size,
                        allocation.height / 2 + self.__pointer_size)

        ctx.stroke_preserve()
        ctx.set_source_rgba(1, 1, 1, 1)
        ctx.fill()

        region = Gdk.cairo_region_create_from_surface(surface)
        self.shape_combine_region(region)

    def draw_top_border(self, drawing_area, event):
        """
            Draw the top of a rectangle with rounded corners to provide
            a border for the window
        """
        # in gtk3 the last param is a cairo context, in gtk2 we need to
        # create one
        if build_gtk2:
            ctx = drawing_area.window.cairo_create()
            ctx.rectangle(event.area.x, event.area.y,
                          event.area.width, event.area.height)
            ctx.clip()
        else:
            ctx = event

        alloc = drawing_area.get_allocation()

        # fill with background the background colour first
        ctx.rectangle(0, 0, alloc.width, alloc.height)
        ctx.set_source_rgb(self.__bgr / 255, self.__bgg / 255,
                           self.__bgb / 255)
        ctx.fill()

        # do the actual drawing
        ctx.set_operator(cairo.OPERATOR_OVER)
        ctx.set_line_width(self.__line_width)
        ctx.set_line_join(cairo.LINE_JOIN_ROUND)
        ctx.set_source_rgb(self.__fgr / 255, self.__fgg / 255,
                           self.__fgb / 255)

        # the position of the top, left and right border lines depend on whether or not we need to
        # shape the window into a pointer to the app icon
        top_extent = left_extent = self.__border_line
        right_extent = alloc.width - self.__border_line
        if self.__do_window_shaping:
            if self.__panel_orient == MatePanelApplet.AppletOrient.DOWN:
                top_extent += self.__pointer_size
            elif self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
                left_extent += self.__pointer_size
            elif self.__panel_orient == MatePanelApplet.AppletOrient.LEFT:
                right_extent -= self.__pointer_size

        ctx.move_to(left_extent, alloc.height)

        ctx.line_to(left_extent, top_extent + self.__line_curve)

        ctx.curve_to(left_extent,
                     top_extent + self.__line_curve,
                     left_extent, top_extent,
                     left_extent + self.__line_curve,
                     top_extent)

        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.DOWN:
            # extend the border line into the pointer
            ctx.line_to(alloc.width / 2 - self.__pointer_size + 1,
                        top_extent)
            ctx.line_to(alloc.width / 2, top_extent - self.__pointer_size + 1)
            ctx.line_to(alloc.width / 2 + self.__pointer_size - 1,
                        top_extent)

        ctx.line_to(right_extent - self.__line_curve, top_extent)
        ctx.curve_to(right_extent - self.__line_curve,
                     top_extent,
                     right_extent, top_extent,
                     right_extent,
                     top_extent + self.__line_curve)
        ctx.line_to(right_extent, alloc.height)
        ctx.stroke()

    def draw_left_border(self, drawing_area, event):
        """
            Draw the left hand side of the window border
        """

        if build_gtk2:
            ctx = drawing_area.window.cairo_create()
            ctx.rectangle(event.area.x, event.area.y,
                          event.area.width, event.area.height)
            ctx.clip()
        else:
            ctx = event

        alloc = drawing_area.get_allocation()

        # fill with background colour
        ctx.rectangle(0, 0, alloc.width, alloc.height)
        ctx.set_source_rgb(self.__bgr / 255, self.__bgg / 255,
                           self.__bgb / 255)
        ctx.fill()

        ctx.set_operator(cairo.OPERATOR_OVER)
        ctx.set_line_width(self.__line_width)
        ctx.set_line_join(cairo.LINE_JOIN_ROUND)
        ctx.set_source_rgb(self.__fgr / 255, self.__fgg / 255,
                           self.__fgb / 255)

        left_extent = self.__border_line
        # the position of the left border depends on whether or not we're doing window shaping
        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
            left_extent += self.__pointer_size

        ctx.move_to(left_extent, 0)

        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
            # extend the border line into the pointer
            ctx.line_to(left_extent,
                        alloc.height / 2 - self.__pointer_size + 1)
            ctx.line_to(self.__border_line, alloc.height / 2)

            ctx.line_to(left_extent, alloc.height / 2 + self.__pointer_size - 1)

        ctx.line_to(left_extent, alloc.height)
        ctx.stroke()

    def draw_right_border(self, drawing_area, event):
        """
            Draw the right hand side of the window border
        """

        if build_gtk2:
            ctx = drawing_area.window.cairo_create()
            ctx.rectangle(event.area.x, event.area.y,
                          event.area.width, event.area.height)
            ctx.clip()
        else:
            ctx = event
        alloc = drawing_area.get_allocation()

        ctx.rectangle(0, 0, alloc.width, alloc.height)
        ctx.set_source_rgb(self.__bgr / 255, self.__bgg / 255,
                           self.__bgb / 255)
        ctx.fill()
        ctx.set_operator(cairo.OPERATOR_OVER)
        ctx.set_line_width(self.__line_width)
        ctx.set_line_join(cairo.LINE_JOIN_ROUND)
        ctx.set_source_rgb(self.__fgr / 255, self.__fgg / 255,
                           self.__fgb / 255)

        right_extent = alloc.width - self.__border_line
        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.LEFT:
            right_extent -= self.__pointer_size

        ctx.move_to(right_extent, 0)

        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.LEFT:
            # extend the border line into the pointer
            ctx.line_to(right_extent,
                        alloc.height / 2 - self.__pointer_size + 1)
            ctx.line_to(alloc.width - self.__border_line, alloc.height / 2)
            ctx.line_to(right_extent, alloc.height / 2 + self.__pointer_size - 1)

        ctx.line_to(right_extent, alloc.height)
        ctx.stroke()

    def draw_bottom_border(self, drawing_area, event):
        """
            Draw the bottom of the window border with rounded corners

        """

        if build_gtk2:
            ctx = drawing_area.window.cairo_create()
            ctx.rectangle(event.area.x, event.area.y,
                          event.area.width, event.area.height)
            ctx.clip()
        else:
            ctx = event

        alloc = drawing_area.get_allocation()
        ctx.rectangle(0, 0, alloc.width, alloc.height)
        ctx.set_source_rgb(self.__bgr / 255, self.__bgg / 255,
                           self.__bgb / 255)
        ctx.fill()

        ctx.set_operator(cairo.OPERATOR_OVER)
        ctx.set_line_width(self.__line_width)
        ctx.set_line_join(cairo.LINE_JOIN_ROUND)
        ctx.set_source_rgb(self.__fgr / 255, self.__fgg / 255,
                           self.__fgb / 255)

        # the lower, right and left extents of the border depend on whether or not we need to
        # shape the window into a pointer to the app icon
        left_extent = self.__border_line
        right_extent = alloc.width - self.__border_line
        lower_extent = alloc.height - self.__border_line
        if self.__do_window_shaping:
            if self.__panel_orient == MatePanelApplet.AppletOrient.UP:
                lower_extent -= self.__pointer_size
            elif self.__panel_orient == MatePanelApplet.AppletOrient.RIGHT:
                left_extent += self.__pointer_size
            elif self.__panel_orient == MatePanelApplet.AppletOrient.LEFT:
                right_extent -= self.__pointer_size

        ctx.move_to(left_extent, 0)
        ctx.line_to(left_extent,
                    lower_extent - self.__line_curve)
        ctx.curve_to(left_extent,
                     lower_extent - self.__line_curve,
                     left_extent, lower_extent,
                     left_extent + self.__line_curve,
                     lower_extent)

        if self.__do_window_shaping and self.__panel_orient == MatePanelApplet.AppletOrient.UP:
            # draw a pointer
            ctx.line_to(alloc.width / 2 - self.__pointer_size + 1,
                        lower_extent)
            ctx.line_to(alloc.width / 2, lower_extent + self.__pointer_size - 1)
            ctx.line_to(alloc.width / 2 + self.__pointer_size - 1,
                        lower_extent)

        ctx.line_to(right_extent - self.__line_curve, lower_extent)

        ctx.curve_to(right_extent - self.__line_curve,
                     lower_extent,
                     right_extent,
                     lower_extent,
                     right_extent,
                     lower_extent - self.__line_curve)
        ctx.line_to(right_extent, 0)
        ctx.stroke()

    def win_hidden(self, widget):
        """ Event handler for the window's hide event

            Delete the timer object

        """

        self.stop_mouse_area_timer()

    def do_timer(self):
        """
            Check the current mouse position and if it is not within any of the
            rectangles in self.__mouse_area hide the window
        """

        # get the mouse x y
        root_win, x, y, mask = self.get_screen().get_root_window().get_pointer()
        if not self.point_is_in_mouse_areas(x, y):
            self.hide()
            self.__timer_id = None
            return False

        return True

    def clear_mouse_areas(self):
        """ Clear the mouse areas list """
        self.__mouse_areas = []

    def add_mouse_area(self, rect):
        """ Add a rectangle to the __mouse_area_list

            Args: rect - a Gdk.Rectangle
        """
        self.__mouse_areas.append(rect)

    def point_is_in_mouse_areas(self, x, y):
        """ Checks to see if a specified position on the screen is within any of
            the self.__mouse_areas rectangle list

            Args:
                x : the x position
                y : the y position

            Returns:
                True if the position is within one of the rectangles in
                self.__mouse_areas, False otherwise
        """

        for rect in self.__mouse_areas:
            if ((x >= rect.x) and (x <= rect.x + rect.width)) and \
               ((y >= rect.y) and (y <= rect.y + rect.height)):
                return True

        return False

    def get_app(self):
        """ Return the docked app the window list refers to

            Returns:
                A docked_app
        """

        return self.__the_app

    def set_app(self, app):
        """ Set the docked app the window list refers to

            Draw the app icon at an appropriate size for the list

            Args : app - a docked_app
        """

        self.__the_app = app
        if self.__icontheme is not None:
            self.get_app_icon()

    the_app = property(get_app, set_app)

    def get_app_icon(self):
        """ Draws the app icon and stores it in self.__app_pb for
            later use

        self.__the_app and the icon theme and size must have been set before
        this is called....
        """
        if self.__icontheme.has_icon(self.__the_app.icon_name):

            # draw the app icon at the size we want
            icon_info = self.__icontheme.choose_icon([self.__the_app.icon_name,
                                                     None],
                                                     self.__icon_size, 0)

            try:
                pixbuf = icon_info.load_icon()
            except GLib.GError:
                # default to a stock icon if we couldn't load the app
                # icon
                pixbuf = self.render_icon(Gtk.STOCK_EXECUTE,
                                          Gtk.IconSize.DND, None)
        else:
            pixbuf = self.the_app.app_pb.scale_simple(self.__icon_size,
                                                      self.__icon_size,
                                                      GdkPixbuf.InterpType.BILINEAR)

        self.__app_pb = pixbuf

    @property
    def bg_col(self):
        return self.__bgr, self.__bgg, self.__bgb

    @property
    def fg_col(self):
        return self.__fgr, self.__fgg, self.__fgb

    @property
    def hl_col(self):
        return self.__hlr, self.__hlg, self.__hlb

    @property
    def icon_size(self):
        return self.__icon_size

    @icon_size.setter
    def icon_size(self, size_in_pixels):
        self.__icon_size = size_in_pixels
        if (self.__the_app is not None) and \
           (self.__icontheme is not None):
            self.__get_app_icon()

    @property
    def app_pb(self):
        return self.__app_pb

    def get_icontheme(self):
        """ Return the icontheme

            Returns:
                A Gtk.Icontheme
        """

        return self.__icontheme

    def set_icontheme(self, the_icontheme):
        """ Sets the icontheme currently being used

            Args : the_icontheme
        """

        self.__icontheme = the_icontheme

    icontheme = property(get_icontheme, set_icontheme)

    def da_pointer_draw(self, drawing_area, event):

        ctx = event
        alloc = drawing_area.get_allocation()

        ctx.rectangle(0, 0, alloc.width, alloc.height)
        ctx.set_source_rgb(self.__bgr / 255, self.__bgg / 255,
                           self.__bgb / 255)
        ctx.fill()

    def set_app_root_coords(self, x, y):
        """ Sets the x and y root coords of the app
        """

        self.__app_x = x
        self.__app_y = y

    def set_applet_details(self, applet_x, applet_y, applet_w, applet_h):
        """ Sets the variables which record the root coords and size of the
            applet

            Args:
                applet_x : the x position of the top left of the applet
                           (root coords)
                applet_y : the y position of the top left of the applet
                           (root coords)
                applet_w : the width of the applet
                applet_h : the height of the applet

        """

        self.__applet_x = applet_x
        self.__applet_y = applet_y
        self.__applet_w = applet_w
        self.__applet_h = applet_h



#!/usr/bin/env python3
"""
Provide hints and tips dialog for the MATE dock applet

The applet displays the following:
        useful hints, tips, and information about use of the dock that
        may not be immediately apparent to the user
        a close button
"""

# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson
#

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
else:
    gi.require_version("Gtk", "3.0")

import sys

from gi.repository import Gtk, Pango


class InfoWindow(Gtk.Window):
    """Provides the Info window"""

    def __init__(self, running_from_about=False):
        """Init for the Info window class

        Create the window and its contents

        Args:
            running_from_about - boolean, if True indicates that the window was
                                 invoked from the About dialog, and therefore
                                 there is no need to show info about how to
                                 show this window from the About dialog...
                                 (needed in case the window ever has to be
                                 invoked by a different method e.g. from the
                                 applet's right click menu)
        """

        super().__init__(title=_("Dock Applet hints, tips, and information"))

        self.__rfa = running_from_about

        self.__button = Gtk.Button(label=_("Close"), stock=Gtk.STOCK_CLOSE)
        self.__button.connect("button-press-event", self.win_button_press)
        self.set_border_width(5)

        if build_gtk2:
            self.__vbox = Gtk.VBox()
        else:
            self.__vbox = Gtk.Box()
            self.__vbox.set_orientation(Gtk.Orientation.VERTICAL)
        self.__vbox.set_spacing(2)

        if build_gtk2:
            self.__hbx = Gtk.HButtonBox()
        else:
            self.__hbx = Gtk.ButtonBox()
            self.__hbx.set_orientation = Gtk.Orientation.HORIZONTAL

        self.__hbx.set_layout(Gtk.ButtonBoxStyle.END)
        self.__hbx.pack_start(self.__button, False, False, 4)

        # create widgets where where the info text can be displayed...
        self.__scrolled_win = Gtk.ScrolledWindow()
        self.__scrolled_win.set_policy(Gtk.PolicyType.NEVER,
                                       Gtk.PolicyType.AUTOMATIC)
        self.__tv_info = Gtk.TextView()
        self.__tv_info.set_wrap_mode(Gtk.WrapMode.WORD)
        self.__tv_info.set_editable(False)
        self.__info_text_buf = Gtk.TextBuffer()
        self.__tag_bold = self.__info_text_buf.create_tag("bold",
                                                          weight=Pango.Weight.BOLD,
                                                          scale=1.3,
                                                          justification=Gtk.Justification.CENTER)
        self.__tv_info.set_buffer(self.__info_text_buf)
        self.set_info_text()
        self.__scrolled_win.add(self.__tv_info)

        self.__vbox.pack_start(self.__scrolled_win, True, True, 4)
        self.__vbox.pack_start(Gtk.HSeparator(), False, False, 2)
        self.__vbox.pack_end(self.__hbx, False, False, 5)
        self.add(self.__vbox)
        self.set_size_request(500, 300)

#        self.set_skip_taskbar_hint(True)
#        self.set_urgency_hint(True)

    def set_info_text(self):
        """ Sets the text which is to be displayed
        """

        the_iter = self.__info_text_buf.get_end_iter()
        self.__info_text_buf.insert_with_tags(the_iter,
                                              "Opening a new instance of a running application" + "\n",
                                              self.__tag_bold)
        self.__info_text_buf.insert(the_iter, "\n" + _("To quickly open a new instance of a running " +
                                              "application either hold down the <shift> key while clicking the " +
                                              "application's dock icon, or middle click on the icon." +
                                              "\n\nNote: this works for most, but not all, apps.") + "\n\n")

        self.__info_text_buf.insert_with_tags(the_iter,
                                              "\n" + _("Window switching using the mouse wheel") + "\n",
                                              self.__tag_bold)
        self.__info_text_buf.insert(the_iter,
                                    "\n" + _("To quickly switch between an application's open windows, move " +
                                    "the mouse cursor over the apps's dock icon and use the mouse " +
                                    "scroll wheel. This will activate and display each window in " +
                                    "turn, changing workspaces as necessary.") + "\n\n")

        self.__info_text_buf.insert_with_tags(the_iter,
                                              "\n" + _("Panel colour changing") + "\n",
                                              self.__tag_bold)
        self.__info_text_buf.insert(the_iter, "\n" + _("When the applet sets the panel colour for the " +
                                    "first time, the result may not look exactly as expected. This is " +
                                    "because the default opacity of custom coloured MATE panels is set " +
                                    "extremely low, so that the panel appears almost transparent.\n\nTo remedy " +
                                    "this, simply right click the panel, select Properties and adjust the " +
                                    "panel opacity as required."))

        if not self.__rfa:
            self.__info_text_buf.insert_with_tags(the_iter,
                                                  "\n" + _("Click the 'Hints & Tips' button on the applet 'About' dialog box "
                                                  "to see this information again."),
                                                  self.__tag_bold)

    def win_button_press(self, widget, event):
        """
        callback for the Ok button on the Info dialog

        The window is hidden so that it can be shown again
        """

        self.destroy()


def main():
    """
    main function - debugging code goes here
    """

    info_win = InfoWindow(True)
    info_win.show_all()
    Gtk.main()

    return


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Provide an application dock applet for the MATE panel

Create a Mate panel applet and handle events generated
by it

Note: Functionality for docked apps is provided in docked_app.py

      Function for the dock is provided in dock.py

"""

# Copyright (C) 1997-2003 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author:
#     Robin Thompson

# do not change the value of this variable - it will be set during build
# according to the value of the --with-gtk3 option used with .configure
build_gtk2 = False

import gi

if build_gtk2:
    gi.require_version("Gtk", "2.0")
    gi.require_version("Wnck", "1.0")
else:
    gi.require_version("Gtk", "3.0")
    gi.require_version("Wnck", "3.0")

gi.require_version("MatePanelApplet", "4.0")

import os
import sys
import threading
sys.path.insert(1, '/usr/lib/python3.14/site-packages')

from Xlib.display import Display
from Xlib import X, error
from gi.repository import Gtk
from gi.repository import MatePanelApplet
from gi.repository import Gdk
from gi.repository import Gio
from gi.repository import GObject
from gi.repository import GLib
from gi.repository import Wnck

import xdg.DesktopEntry as DesktopEntry
from urllib.parse import urlparse

import docked_app
import dock

from log_it import log_it as log_it

drag_dropped = False   # nasty global var used to keep track of whether or not a drag-drop event has occurred

# define a list of keyboard shortcuts to be used to activate specific apps in the dock
# '<Super>1' to '<Super>0' will correspond to apps 1 to 10
# '<Super><Alt>1' to '<Super><Alt>9' will correspond to apps 11 to 20
keyb_shortcuts = ["<Super>1", "<Super>2", "<Super>3", "<Super>4", "<Super>5",
                  "<Super>6", "<Super>7", "<Super>8", "<Super>9", "<Super>0",
                  "<Super><Alt>1", "<Super><Alt>2", "<Super><Alt>3", "<Super><Alt>4", "<Super><Alt>5",
                  "<Super><Alt>6", "<Super><Alt>7", "<Super><Alt>8", "<Super><Alt>9", "<Super><Alt>0"]


def applet_button_press(widget, event, the_dock):
    """Button press event for the applet

    Handle right button press events only

    Find the app that was right clicked and make a record of it

    Args:
        widget : the widget that was clicked
        event : the event args
        the_dock : the Dock object
    """

    # we don't get click events for the right mouse button presumably
    # because the panel hijacks them in order to produce the context menu
    # However, we do get button press event for the right mouse button,
    # so we can do what we need to do here ....
    if event.button == 3:
        # right click, so save the app that was clicked because
        # the_dock.app_with_mouse is going to be set to None when the
        # right click menu appears and we move the mouse over the menu to
        # select an option
        app = the_dock.get_app_at_mouse(event.x, event.y)
        the_dock.right_clicked_app = app

        # because the right click menu is about to be shown, we need to hide
        # the window list
        the_dock.hide_win_list()
        the_dock.hide_act_list()
    elif event.button == 1:

        if the_dock.panel_expand is not True:
            # prevent the panel acting on the button press that we got
            # e.g. so that dragging dock icons does not also start a drag of the panel
            GObject.signal_stop_emission_by_name(widget, "button_press_event")

        dx, dy = the_dock.get_drag_coords()
        if (dx == -1) and (dy == -1):
            the_dock.set_drag_coords(event.x, event.y)


def applet_button_release(widget, event, the_dock):
    """Button press event for the applet

    Handle left button release events only

    If the button is released over a non-running app, start the app

    If the button is released over a running app that isn't on the
    current workspace, change workspace

    If the button is released over a running app:
        If the app has only a single window open, activate it
        If window list is showing, hide it
        If the window list is not visible, show it

    Since the button has been released, make sure that
    icon dragging doesn't occur

    Args:
        widget : the widget that registered the release event
        event : the event args
        the_dock : the Dock object

    """

    if event.button == 1:
        the_dock.clear_drag_coords()

        # hide popups
        the_dock.hide_act_list()

        app = the_dock.get_app_at_mouse(event.x, event.y)
        if app is not None:

            start_app = app.is_running() is False
            start_app = start_app | (event.state &
                                     Gdk.ModifierType.SHIFT_MASK) != 0
            if start_app:
                app.start_app()
            else:
                if the_dock.win_switch_unity_style:
                    if app.is_active and app.get_num_windows() > 1:
                        the_dock.do_window_selection(app)
                    else:
                        the_dock.minimize_or_restore_windows(app, True)
                else:
                    # if the app only has a single window minimize or restore it, otherwise
                    # perform the action specified by the user
                    if app.get_num_windows() == 1:
                        the_dock.minimize_or_restore_windows(app)
                    else:
                        the_dock.do_window_selection(app)


    # See https://bugs.launchpad.net/ubuntu-mate/+bug/1554128
    if event.button == 2:
        app = the_dock.get_app_at_mouse(event.x, event.y)
        if app is not None:
            the_dock.hide_win_list()
            the_dock.hide_act_list()
            app.start_app()


def applet_enter_notify(widget, event, the_dock):
    """Enter notify event for the applet

    Brighten the icon of the app which the mouse is currently over

    If another app is currently brightened, darken it to normal

    Set up the right click menu for the dock based on the app which
    the mouse is currently over

    Start the timer for showing app window lists

    Args:
        widget : the widget that registered the event i.e. the applet
        event : the event args
        the_dock : the Dock object
    """

    # get the app underneath the mouse cursor
    app = the_dock.get_app_at_mouse(event.x, event.y)

    # if an app is currently highlighted, de-highlight it
    if the_dock.app_with_mouse is not None:

        the_dock.app_with_mouse.has_mouse = False
        the_dock.app_with_mouse.queue_draw()
        the_dock.app_with_mouse = None

    # highlight the app under the mouse cursor
    if app is not None:
        app.has_mouse = True
        app.queue_draw()

        the_dock.app_with_mouse = app

        # set up the available options for the app
        the_dock.set_actions_for_app(app)
    else:
        the_dock.app_with_mouse = None


def applet_leave_notify(widget, event, the_dock):
    """Leave notify event handle for the applet

    Unbright any brightened app icon

    Args:
        widget : the widget that registered the event i.e. the applet
        event : the event args
        the_dock : the Dock object
    """

    if the_dock.app_with_mouse is not None:
        the_dock.app_with_mouse.has_mouse = False
        the_dock.app_with_mouse.queue_draw()
        the_dock.app_with_mouse = None

        the_dock.stop_act_list_timer()
        if the_dock.scrolling:
            the_dock.stop_scroll_timer()


def applet_motion_notify(widget, event, the_dock):
    """Motion notify event for the applet

    If the docked app under the mouse cursor does not have its icon
    brightened and another app has a brightened icon then darken the other app
    # icon and reset the applet tooltip text

    Then, if the docked app under the mouse cursor does not have its icon
    brightened then brighten it and setup the applet right click menu

    Args:
        widget : the widget that registered the event i.e. the applet
        event : the event args
        the_dock : the Dock object
    """

    app = the_dock.get_app_at_mouse(event.x, event.y)

    if (the_dock.app_with_mouse is not None) and \
       (the_dock.app_with_mouse != app):
        the_dock.app_with_mouse.has_mouse = False
        the_dock.app_with_mouse.queue_draw()

        widget.queue_draw()

        # because a new app is highlighted reset the window list timer and hide
        # any currently open window list and action list
        the_dock.hide_win_list()
        the_dock.reset_act_list_timer()
        the_dock.hide_act_list()

    if app is not None:

        the_dock.app_with_mouse = app

        # reset the window list timer
        the_dock.reset_act_list_timer()

        if the_dock.scrolling and app.scroll_dir != docked_app.ScrollType.SCROLL_NONE:
            the_dock.reset_scroll_timer()

        if app.has_mouse is False:
            app.has_mouse = True
            app.queue_draw()
            the_dock.app_with_mouse = app
            the_dock.set_actions_for_app(app)

    else:
        the_dock.app_with_mouse = None

        the_dock.set_actions_for_app(None)

    dx, dy = the_dock.get_drag_coords()
    if (dx != -1) and (dy != -1) and not the_dock.dragging:
        # we may need to begin a drag operation

        if widget.drag_check_threshold(dx, dy, event.x, event.y):
            target_list = widget.drag_dest_get_target_list()
            context = widget.drag_begin_with_coordinates(target_list,
                                                         Gdk.DragAction.MOVE, 1,
                                                         event, -1, -1)

            applet_drag_begin(widget, context, the_dock)


def applet_change_orient(applet, orient, the_dock):
    """Handler for applet change orientation event

    Set the dock to the new orientation and re-show the applet

    Args:
        applet : the widget that registered the event i.e. the applet
        orient : the new orientation
        the_dock : the Dock object
    """

    the_dock.set_new_orientation(orient)
    the_dock.applet.show_all()
    the_dock.show_or_hide_app_icons()


def applet_size_allocate(applet, allocation, the_dock):
    """ When the applet can play nicely with panel, ensure that it
        fits within the allocated space


    Args :
        applet : the applet
        allocation : a Gtk.Allocation - the space in which the applet must
                     fit
        the_dock : the Dock object
    """

    if the_dock.nice_sizing:
        the_dock.fit_to_alloc()
    return


def applet_change_size(applet, size, the_dock):
    """Handler for the applet change size event

    Resize the icon and recalculate the minimize location of each app in the
    dock

    Args:
        applet : the widget that registered the event i.e. the applet
        size : the new applet size
        the_dock : the Dock object
    """

    for app in the_dock.app_list:
        the_dock.set_app_icon(app, size)


def applet_scroll_event(applet, event, the_dock):
    """ Handler for the scroll event

    Call the dock's  function to move forward/backward through the active app's
    windows

    """

    # with V0.81 the dock contains a scrolled window and we now only get
    # a ScrollDirection of SMOOTH here ....
    if event.direction == Gdk.ScrollDirection.SMOOTH:
        hasdeltas, dx, dy = event.get_scroll_deltas()
        if dy < 0:
            the_dock.do_window_scroll(Gdk.ScrollDirection.DOWN, event.time)
        elif dy > 0:
            the_dock.do_window_scroll(Gdk.ScrollDirection.UP, event.time)


def applet_drag_begin(applet, context, the_dock):
    """
        Let the dock know we're dragging an icon.
        Redraw the icon of the app that's being dragged so that the user has
        visual feedback that the drag has started
        Set the drag cursor to the app icon
        Start a timer to monitor the mouse x,y and move the dragged app icon
        around the dock accordingly

    """

    # we can sometimes get spurious applet-leave events just before a drag
    # commences. This causes app_with_mouse to be set to None. Therefore we
    # may need to identify the app under the mouse ourselves...

    if the_dock.app_with_mouse is None:
        the_dock.app_with_mouse = the_dock.get_app_under_mouse()

    if the_dock.app_with_mouse is not None:
        the_dock.app_with_mouse.set_dragee(True)
        the_dock.app_with_mouse.queue_draw()

        Gtk.drag_set_icon_pixbuf(context, the_dock.app_with_mouse.app_pb,
                                 0, 0)

        the_dock.start_drag_motion_timer(the_dock.app_with_mouse)
        the_dock.dragging = True

        # finally, hide the window list if it was being shown
        the_dock.hide_win_list()
        the_dock.hide_act_list()
        the_dock.stop_scroll_timer()


def applet_drag_data_get(widget, drag_context, data, info, time):
    """
        Handler the for drag-data-get event

        Set some dummy text as data for the drag and drop
    """

    data.set_text("", -1)


def applet_drag_drop(widget, context, x, y, time, the_dock):
    """
        Handler for the drag-drop event

        The drag drop is over so:
            Call Gtk.drag-finish and indicate the drag and drop completed ok
            Let the dock know that the drag and drop has finished and redraw
            the dragged app's icon
            Stop the timer that monitors the mouse position
    """

    app = the_dock.get_dragee()
    if app is not None:
        the_dock.stop_drag_motion_timer()
        app.set_dragee(False)
        app.queue_draw()
        Gtk.drag_finish(context, True, False, time)
    else:
        # set the drag_dropped module level var so that the drag_data_received event knows
        # the dnd needs to finish
        global drag_dropped
        drag_dropped = True

        target = widget.drag_dest_find_target(context, None)
        widget.drag_get_data(context, target, time)
        return True


def applet_drag_data_received(widget, drag_context, x, y, data, info, time, the_dock):
    """ Called when data has been requested from an external source
        during a drag drop operation

    Examine the data - if it is a .desktop file and the app it relates to
    is not already in the dock, add it. If the data isn't a .desktop file
    and the app under the mouse cursor is running, activate it so that
    the dragged data can be dropped there...

    :param widget: the widget responsible for the event
    :param drag_context: the dnd context
    :param x: the x position of the mouse
    :param y:  y the y position of the mouse
    :param data: the dragged data
    :param info:
    :param time:  the time of the event
    :param the_dock: the dock ....
    """

    # examine the data  -did we get any uris ?
    uri_list = data.get_uris()
    if (uri_list is not None) and (len(uri_list) > 0):
        # when dragging .desktop files to the dock we only allow one to be added at
        # a time. Therefore we're only interested in the first item in the list
        uri = urlparse(uri_list[0])
        if uri.scheme == "file":
            # we're looking for a .desktop file
            if (uri.path != "") and (os.path.split(uri.path)[1].endswith(".desktop")) and \
               (os.path.exists(uri.path)):

                # we've got a .desktop file, so if it has been dropped we may need
                # to add it to the dock
                global drag_dropped
                if drag_dropped:
                    # add the .desktop file to the dock if it is not already there,,,
                    the_dock.add_app_to_dock(uri.path)

                    # cancel the dnd
                    Gtk.drag_finish(drag_context, True, False, time)
                    drag_dropped = False
                    return
                else:
                    # the dnd continues ....
                    Gdk.drag_status(drag_context, Gdk.DragAction.COPY, time)
                    return

    # this is not a .desktop so we need to activate the app under the mouse
    tgt_app = the_dock.get_app_under_mouse()
    the_dock.start_da_timer(tgt_app)
    Gdk.drag_status(drag_context, Gdk.DragAction.COPY, time)


def applet_drag_end(widget, context, the_dock):
    """
    Handler for the drag-end event

    This will be triggered when e.g. the use drags an icon off the panel and
    releases the mouse button ....
    Let the dock know that the drag and drop has finished and redraw the
    dragged app's icon
    Stop the timer that monitors the mouse position
    """

    the_dock.stop_drag_motion_timer()

    app = the_dock.get_dragee()
    if app is not None:
        app.set_dragee(False)
        app.queue_draw()

    the_dock.dragging = False
    the_dock.clear_drag_coords()


def applet_drag_motion(widget, context, x, y, time, the_dock):
    """ Handler for the drag-motion event

    :param widget:  - the applet
    :param context: - the dnd context
    :param x:       - x coord of the mouse
    :param y:       - y coord of the mouse
    :param time:    - the time of the event
    :param the_dock - the dock
    :return:
    """

    # if the applet isn't dragging an app icon, we may need to examine
    # the dragged data to see what we are dragging
    app = the_dock.get_dragee()
    if app is None:
        # examine the dragged data so we can decide what to do...
        tgts = context.list_targets()
        for t in tgts:
            if t.name() == "text/uri-list":
                # if the data contains uris, we need to request the data to
                # see if it contains a .desktop file
                widget.drag_get_data(context, t, time)
                return True

        # if the dragged data is anything other than a uri, we just need to activate the app under
        # the mouse...
        tgt_app = the_dock.get_app_under_mouse()
        the_dock.start_da_timer(tgt_app)
        return True
    else:
        # continue the dnd...
        Gdk.drag_status(context, Gdk.DragAction.COPY, time)

    return True


def applet_shortcut_handler(keybinder, the_dock):
    """ Handler for global keyboard shortcut presses

    Start the app if it isn't already running

    If it is already runnning cycle through its windows ...

    :param keybinder: the keybinder object with the keystring which was pressed e.g. "<Super>4"
    :param the_dock: the dock...
    """
    # get the position in the dock of the app we need to activate
    if keybinder.current_shortcut in keybinder.shortcuts:
        app_no = keybinder.shortcuts.index(keybinder.current_shortcut)

    app = the_dock.get_app_by_pos(app_no)
    if app is not None:
        start_app = app.is_running() is False
        if start_app:
            app.start_app()
        else:

            # if the app only has a single window minimize or restore it
            # otherwise scroll through all available windows
            if app.get_num_windows() == 1:
                the_dock.minimize_or_restore_windows(app)
            else:
                the_dock.do_window_scroll(Gdk.ScrollDirection.DOWN, 0, app)


def applet_fill(applet):
    """
    Create the applet

    Register the events that we're interested in getting events for and
    connect event handlers for them

    Create a dock and add it V/HBox to the applet


    Args:
        applet : the applet
    """

    os.chdir(os.path.expanduser("~"))

    applet.set_events(applet.get_events() |
                      Gdk.EventMask.BUTTON_PRESS_MASK |
                      Gdk.EventMask.BUTTON_RELEASE_MASK |
                      Gdk.EventMask.POINTER_MOTION_MASK |
                      Gdk.EventMask.KEY_PRESS_MASK |
                      Gdk.EventMask.KEY_RELEASE_MASK |
                      Gdk.EventMask.SCROLL_MASK |
                      Gdk.EventMask.STRUCTURE_MASK)

    the_dock = dock.Dock(applet)
    the_dock.setup_dock()

    if the_dock.nice_sizing:
        applet.set_flags(MatePanelApplet.AppletFlags.EXPAND_MAJOR |
                         MatePanelApplet.AppletFlags.EXPAND_MINOR |
                         MatePanelApplet.AppletFlags.FLAGS_NONE)
    else:
        applet.set_flags(AppletFlags.FLAGS_NONE)

    if build_gtk2:
        applet.add(the_dock.box)
    else:
        applet.add(the_dock.scrolled_win)

    applet.show_all()

    # make sure that apps pinned to specific workspaces other than the current one
    # are hidden
    the_dock.show_or_hide_app_icons()

    applet.connect("enter-notify-event", applet_enter_notify, the_dock)
    applet.connect("leave-notify-event", applet_leave_notify, the_dock)
    applet.connect("motion-notify-event", applet_motion_notify, the_dock)
    applet.connect("button-press-event", applet_button_press, the_dock)
    applet.connect("button-release-event", applet_button_release, the_dock)
    applet.connect("change-orient", applet_change_orient, the_dock)
    applet.connect("change-size", applet_change_size, the_dock)
    applet.connect("scroll-event", applet_scroll_event, the_dock)
    applet.connect("size-allocate", applet_size_allocate, the_dock)

    if not build_gtk2:
        # set up drag and drop - gtk3 only
        # NOTE: we don't get drag-motion events when dragging app icons within the
        # dock, making it difficult to tell where the mouse pointer is.....
        # To get around this, dock.py now contains a timer to monitor the
        # mouse x.y during these sorts of drag and drops.
        # drag-motion events do fire when dropping from other apps (e.g. caja)
        # and the drag-motion event is used in these cases

        # we allow .desktop files to be dropped on the applet, so....
        drag_tgts = [Gtk.TargetEntry.new("text/uri-list", 0, 0)]
        applet.drag_source_set(Gdk.ModifierType.BUTTON1_MASK, drag_tgts,
                               Gdk.DragAction.MOVE)

        drag_tgts = [Gtk.TargetEntry.new("text/uri-list", 0, 0)]
        applet.drag_dest_set(Gtk.DestDefaults.MOTION, None, Gdk.DragAction.COPY)
        applet.drag_dest_add_image_targets()
        applet.drag_dest_add_text_targets()
        applet.drag_dest_add_uri_targets()

        applet.connect("drag-data-get", applet_drag_data_get)
        applet.connect("drag-drop", applet_drag_drop, the_dock)
        applet.connect("drag-end", applet_drag_end, the_dock)
        applet.connect("drag-motion", applet_drag_motion, the_dock)
        applet.connect("drag-data-received", applet_drag_data_received, the_dock)

    # set up keyboard shortcuts used to activate apps in the dock
    keybinder = GlobalKeyBinding()
    for shortcut in keyb_shortcuts:
        keybinder.grab(shortcut)
    keybinder.connect("activate", applet_shortcut_handler, the_dock)
    keybinder.start()

    applet.set_background_widget(applet)  # hack for panel transparency


def applet_factory(applet, iid, data):
    """Factory routine called when an applet needs to be created

    Create a dock applet if necessary

    Args:
        applet : the applet
        iid    : the id of the applet that needs to be created
        data   :
    Returns:
        True if we created a dock applet, False otherwise
    """

    if iid != "DockApplet":
        return False

    applet_fill(applet)

    return True


class GlobalKeyBinding(GObject.GObject, threading.Thread):
    __gsignals__ = {
        'activate': (GObject.SignalFlags.RUN_LAST, None, ()),
    }

    def __init__(self):
        GObject.GObject.__init__(self)
        threading.Thread.__init__(self)
        self.setDaemon(True)

        self.display = Display()
        self.screen = self.display.screen()
        self.window = self.screen.root
        self.keymap = Gdk.Keymap().get_default()
        self.ignored_masks = self.get_mask_combinations(X.LockMask | X.Mod2Mask | X.Mod5Mask)
        self.map_modifiers()
        self.shortcuts = []

    def get_mask_combinations(self, mask):
        return [x for x in range(mask + 1) if not (x & ~mask)]

    def map_modifiers(self):
        gdk_modifiers = (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.SHIFT_MASK, Gdk.ModifierType.MOD1_MASK,
                         Gdk.ModifierType.MOD2_MASK, Gdk.ModifierType.MOD3_MASK, Gdk.ModifierType.MOD4_MASK, Gdk.ModifierType.MOD5_MASK,
                         Gdk.ModifierType.SUPER_MASK, Gdk.ModifierType.HYPER_MASK)
        self.known_modifiers_mask = 0
        for modifier in gdk_modifiers:
            if "Mod" not in Gtk.accelerator_name(0, modifier) or "Mod4" in Gtk.accelerator_name(0, modifier):
                self.known_modifiers_mask |= modifier

    def idle(self):
        self.emit("activate")
        return False

    def activate(self):
        GLib.idle_add(self.run)

    def grab(self, shortcut):
        keycode = None
        accelerator = shortcut.replace("<Super>", "<Mod4>")
        keyval, modifiers = Gtk.accelerator_parse(accelerator)

        try:
            keycode = self.keymap.get_entries_for_keyval(keyval).keys[0].keycode
        except AttributeError:
            # In older Gtk3 the get_entries_for_keyval() returns an unnamed tuple...
            keycode = self.keymap.get_entries_for_keyval(keyval)[1][0].keycode
        modifiers = int(modifiers)
        self.shortcuts.append([keycode, modifiers])

        # Request to receive key press/release reports from other windows that may not be using modifiers
        catch = error.CatchError(error.BadWindow)
        self.window.change_attributes(onerror=catch, event_mask=X.KeyPressMask)
        if catch.get_error():
            return False

        catch = error.CatchError(error.BadAccess)
        for ignored_mask in self.ignored_masks:
            mod = modifiers | ignored_mask
            result = self.window.grab_key(keycode, mod, True, X.GrabModeAsync, X.GrabModeAsync, onerror=catch)
        self.display.flush()
        if catch.get_error():
            return False
        return True

    def run(self):
        self.running = True
        while self.running:
            event = self.display.next_event()
            if (hasattr(event, 'state')):
                modifiers = event.state & self.known_modifiers_mask
                self.current_shortcut = None
                if event.type == X.KeyPress and [event.detail, modifiers] in self.shortcuts:
                    # Track this shortcut to know which app to activate
                    self.current_shortcut = [event.detail, modifiers]
                    GLib.idle_add(self.idle)
                    self.display.allow_events(X.AsyncKeyboard, event.time)
                else:
                    self.display.allow_events(X.ReplayKeyboard, event.time)

    def stop(self):
        self.running = False
        self.ungrab()
        self.display.close()

    def ungrab(self):
        for shortcut in self.shortcuts:
            self.window.ungrab_key(shortcut[0], X.AnyModifier, self.window)


MatePanelApplet.Applet.factory_main("DockAppletFactory", True,
                                    MatePanelApplet.Applet.__gtype__,
                                    applet_factory, None)


def main():
    """Main function.

    Debugging code can go here
    """
    pass


if __name__ == "__main__":
    main()
